//
// C++ Interface: kpgaggregate
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef KPGAGGREGATE_H
#define KPGAGGREGATE_H

#include "kpgtreeitem.h"

class KPGAggregatesFolder;

/**
Aggregate function. An aggregate function is a function that operates on a set of values (typically one column from each row that matches a query condition) and returns a single value computed from all these values. Typical aggregate functions are sum, count, and max. Each entry in pg_aggregate is an extension of an entry in pg_proc. The pg_proc entry carries the aggregate's name, input and output data types, and other information that is similar to ordinary functions. 

@author Lumir Vanek
*/

class KPGAggregate : public KPGTreeItem
{
public:
    KPGAggregate(KPGAggregatesFolder *, const QString);
		KPGAggregate(KPGAggregatesFolder *, KPGAggregate *, const QString);
    ~KPGAggregate();

virtual eNodeType type() const { return nodeAggregate; }
		void setProperties(const QString, pqxx::oid, const QString, const QString, pqxx::oid, const QString, pqxx::oid, const QString, pqxx::oid, const QString, pqxx::oid, const QString, pqxx::oid, const QString, const QString);
		void displayProperties(QWidget *);
		virtual bool refresh() { return true; }
		
protected:
		pqxx::oid m_oidAggregateFn;
		QString m_strAggregateFn;
		QString m_strDescription;
		QString m_strOwner;
		pqxx::oid m_oidTransFn;
		QString m_strTransFn;
		pqxx::oid m_oidFinalFn;
		QString m_strFinalFn;
		
		pqxx::oid m_oidInputType;
		QString m_strInputTypName;
		pqxx::oid m_oidTransType;
		QString m_strTransTypName;
		pqxx::oid m_oidRetType;
		QString m_strRetTypName;
		
		QString m_strInitVal;
};

#endif
