//
// C++ Implementation: kpgconversionsfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgconversionsfolder.h"

#include <qtable.h>

#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgconnection.h"
#include "kpgconversion.h"
#include "kpgschema.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "../kpgsqldialog.h"

KPGConversionsFolder::KPGConversionsFolder(KPGSchema *parent, pqxx::oid oidNameSpace)
 : KPGItemsFolder(parent, i18n("Conversions"))
{
	setPixmap(0, *m_pIconFolderGrey);
	m_oidNameSpace = oidNameSpace;
}


KPGConversionsFolder::~KPGConversionsFolder()
{
}
bool KPGConversionsFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items 
  while (QListViewItem * pItem = firstChild())
    delete pItem;

	// obtain list of conversions
  QString strQuery("SELECT co.oid, co.conname, co.condefault, pg_encoding_to_char(conforencoding) as fromencoding, pg_encoding_to_char(contoencoding) as toencoding, conproc::oid AS conprocoid, pr.proname, pg_get_userbyid(conowner) as owner ");
		
	strQuery.append("FROM pg_catalog.pg_conversion co ");
	strQuery.append("JOIN pg_catalog.pg_proc pr ON pr.oid=conproc ");
	strQuery.append("JOIN pg_catalog.pg_namespace na ON na.oid=pr.pronamespace ");
	strQuery.append("WHERE connamespace = " + QString("%1").arg(m_oidNameSpace) + " ");
	strQuery.append("ORDER BY conname");
			
  try
  {
    m_pqxxResultConversions = pqxxXaction.Exec(strQuery.latin1());

		KPGConversion *pgConversion = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultConversions.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultConversions[i][0].to(oid);
		
      if(pgConversion == 0)
				pgConversion = new KPGConversion(this, m_pqxxResultConversions[i]["conname"].c_str(), oid);
			else
				pgConversion = new KPGConversion(this, pgConversion, m_pqxxResultConversions[i]["conname"].c_str(), oid);
			
			bool m_bIsDefault;
			pqxx::oid oidConversionProcedure = 0;
			
			m_pqxxResultConversions[i]["condefault"].to(m_bIsDefault);
			m_pqxxResultConversions[i]["conprocoid"].to(oidConversionProcedure);
						
			pgConversion->setProperties(
				m_bIsDefault,
				m_pqxxResultConversions[i]["fromencoding"].c_str(), 
				m_pqxxResultConversions[i]["toencoding"].c_str(), 
				oidConversionProcedure,
				m_pqxxResultConversions[i]["proname"].c_str(), 
				m_pqxxResultConversions[i]["owner"].c_str()
				); 
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGConversionsFolder::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  	pPropertyPage->setLabelText(i18n("Conversions &list:"));
	
	//----------------------------------------------------------------------------
	//Display conversions list
		
	QTable *pTableConversions = pPropertyPage->m_pTableItems;
  
  // fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultConversions, pTableConversions);

	// field names
	QHeader* hHeader = pTableConversions->horizontalHeader();
	
	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Is default"));
	hHeader->setLabel(nCol++, i18n("Source encoding"));
	hHeader->setLabel(nCol++, i18n("Destination encoding"));
 	hHeader->setLabel(nCol++, i18n("Conversion procedure"));
	hHeader->setLabel(nCol++, i18n("Owner"));


  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableConversions->adjustColumn(nCol);
  }
}



