//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgdomainconstraintsfolder.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "kpgconnection.h"
#include "kpgdomain.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "kpgdomainconstraint.h"
#include "../kpgsqldialog.h"


KPGDomainConstraintsFolder::KPGDomainConstraintsFolder(KPGDomain *parent)
 : KPGTreeItem(parent, i18n("Constraints"))
{
  setPixmap(0, *m_pIconFolderGreen);
}


KPGDomainConstraintsFolder::~KPGDomainConstraintsFolder()
{
}

bool KPGDomainConstraintsFolder::refresh()
{
  KPGTreeItem * pItemParent = static_cast <KPGTreeItem *> (parent());
  if(!pItemParent)
    { 
      kdError() << "Failed to find DB connection " << endl;
      return 0;
    }
  
  KPGConnection *pDbConnection = connection();
  work pqxxXaction(*pDbConnection, "refresh");
    
  return refresh(pqxxXaction, (static_cast <KPGDomain *> (pItemParent))->oid());
}

bool KPGDomainConstraintsFolder::refresh(work &pqxxXaction, pqxx::oid oidDomain)
{
  // delete all child items
  while (QListViewItem * pItem = firstChild())
    delete pItem;

  QString strQuery("SELECT c.oid, conname, consrc, description, nspname "); 
  
  strQuery.append("FROM pg_catalog.pg_constraint c ");

  strQuery.append("JOIN pg_catalog.pg_type ty ON ty.oid=contypid ");
  strQuery.append("JOIN pg_catalog.pg_namespace nl ON nl.oid=connamespace ");
  strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON des.objoid=c.oid ");
  
  strQuery.append("WHERE contypid = ");
  strQuery.append(QString("%1").arg(oidDomain));
  strQuery.append(" ORDER BY conname;");
    
  try
  {
    m_pqxxResultConstraints = pqxxXaction.exec(strQuery.latin1());
    
    KPGDomainConstraint * pDomainConstraint = 0;
    
    for (result::size_type i = 0; i != m_pqxxResultConstraints.size(); ++i)
    {
      pqxx::oid oid;
      m_pqxxResultConstraints[i][0].to(oid);
      
      if(pDomainConstraint == 0)
        pDomainConstraint = new KPGDomainConstraint(this, m_pqxxResultConstraints[i]["conname"].c_str(), oid);
      else
        pDomainConstraint = new KPGDomainConstraint(this, pDomainConstraint, m_pqxxResultConstraints[i]["conname"].c_str(), oid);
        
      pDomainConstraint-> setProperties(
        m_pqxxResultConstraints[i]["description"].c_str(), 
        m_pqxxResultConstraints[i]["consrc"].c_str(), 
        m_pqxxResultConstraints[i]["nspname"].c_str()
        );
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
    dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGDomainConstraintsFolder::displayProperties(QWidget *widgetProperies)
{
  if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
	pPropertyPage->setLabelText(i18n("Constraints &list:"));
	
	//----------------------------------------------------------------------------
	//Display constraints list
	QTable *pTableItems = pPropertyPage->m_pTableItems;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultConstraints, pTableItems);
	
	// field names
	QHeader* hHeader = pTableItems->horizontalHeader();
	
	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Definition"));
	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Namespace"));
	
	for(int nCol = 0; nCol < nTableCols; nCol++)
	{
		pTableItems->adjustColumn(nCol);
	}
}
