//
// C++ Implementation: kpgfunction
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgfunction.h"

#include <qlineedit.h>
#include <qtable.h>

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktextedit.h>

#include "kpgserver.h"
#include "kpgdatabase.h"
#include "../kpglinklabel.h"
#include "kpgfunctionsfolder.h"
#include "../PropertyPages/kpgfunctionproppage.h"
#include "../kpgsqldialog.h"


KPGFunction::KPGFunction(KPGFunctionsFolder *parent, const QString name, pqxx::oid _oid)
 : KPGObject(parent, name, _oid)
{
	setPixmap(0, *m_pIconFunction);
}

KPGFunction::KPGFunction(KPGFunctionsFolder *parent, KPGFunction *after, const QString name, pqxx::oid _oid)
 : KPGObject(parent, after, name, _oid)
{
	setPixmap(0, *m_pIconFunction);
}


KPGFunction::~KPGFunction()
{
}

void KPGFunction::setProperties(
		const QString strDescription,
		bool bIsAgg,
		bool bIsSecdef,
		bool bIsStrict,
		bool bRetSet,
		const QString strVolatile,
		int iNArgs,
		const QString strBin,
		const QString strACL,
		const QString strReturnTypName,
		pqxx::oid oidReturnType,
		const QString strLanName,
		const QString strFuncOwner
	)
{
	m_strDescription = strDescription;
	m_bIsAgg = bIsAgg;
	m_bIsSecdef = bIsSecdef;
	m_bIsStrict = bIsStrict;
	m_bRetSet = bRetSet;
	m_strVolatile = strVolatile;
	m_iNArgs = iNArgs;
	m_strBin = strBin;
	m_strACL = strACL;
	m_strReturnTypName = strReturnTypName;
	m_oidReturnType = oidReturnType;
	m_strLanName = strLanName;
	m_strFuncOwner = strFuncOwner;
	
	//--- Set icon for function type
	if(strLanName == "c")
		setPixmap(0, *m_pIconFunctionC);
	else
		if(strLanName == "sql")
			setPixmap(0, *m_pIconFunctionSql);
		else
			if(strLanName == "plpgsql")
				setPixmap(0, *m_pIconFunctionPl);
}

// Set arguments list. All parameters come in form: {aaa, bbb, ccc}
void KPGFunction::setListArguments(const QString &strArgTypes, const QString &strAllArgTypes, const QString &strArgModes, const QString &strArgNames, pqxx::work &pqxxXaction)
{
	m_listArguments.clear();
	
	QStringList strListNames;
	if(strArgNames.length() > 2)
	 	strListNames = QStringList::split (",", strArgNames.mid(1, strArgNames.length() - 2), true);
	
	QStringList strListModes;
	if(strArgModes.length() > 2)
	 	strListModes = QStringList::split (",", strArgModes.mid(1, strArgModes.length() - 2), true);
	
	// Use m_strArgTypes, when strListAllArgTypes is empty
	QStringList strListArgTypesOids;
	if(strAllArgTypes.length() > 2)
       strListArgTypesOids = QStringList::split (",", strAllArgTypes.mid(1, strAllArgTypes.length() - 2), true);
	else
		strListArgTypesOids = QStringList::split (" ", strArgTypes);		
	
	// Number of all arguments - max list size
	unsigned int nArguments = m_iNArgs;
	if(strListNames.count() > nArguments) nArguments = strListNames.count();
	if(strListModes.count() > nArguments) nArguments = strListModes.count();
	if(strListArgTypesOids.count() > nArguments) nArguments = strListArgTypesOids.count();
	
	if((strListNames.count() > 0) && (nArguments != strListNames.count())) 
	{
		kdDebug() << text(0) << QString(" - Names count %1, num of args %2").arg(strListNames.count()).arg(nArguments) << endl;
		kdDebug() << "strAllArgTypes " << strAllArgTypes << endl;
		kdDebug() << "strArgModes " << strArgModes << endl;
		kdDebug() << "strArgNames " << strArgNames << endl;
	}
	if((strListModes.count() > 0) && (nArguments != strListModes.count())) 
	{
		kdDebug() << text(0) << QString(" - Modes count %1, num of args %2").arg(strListModes.count()).arg(nArguments) << endl;
		kdDebug() << "strAllArgTypes " << strAllArgTypes << endl;
		kdDebug() << "strArgModes " << strArgModes << endl;
		kdDebug() << "strArgNames " << strArgNames << endl;
	}
	if((strListArgTypesOids.count() > 0) && (nArguments != strListArgTypesOids.count())) 
	{
		kdDebug() << text(0) << QString(" - Types count %1, num of args %2").arg(strListArgTypesOids.count()).arg(nArguments) << endl;
		kdDebug() << "strArgTypes " << strArgTypes << endl;
		kdDebug() << "strAllArgTypes " << strAllArgTypes << endl;
		kdDebug() << "strArgModes " << strArgModes << endl;
		kdDebug() << "strArgNames " << strArgNames << endl;
	}
	
	for(unsigned int i = 0; i < nArguments; i++)
	{
        QString strName;
         
        if(strListNames.count() > 0)
        { 
         	strName = strListNames[i]; 
			if(strName == "\"\"") strName.truncate(0);
        }
        
        QString strMode;
        if(strListModes.count() > 0)
        {
        	strMode = strListModes[i];
        }
        
        QString strArgTypeOid;
        QString strArgTypeName;
        if(strListArgTypesOids.count() > 0)
        {
        	strArgTypeOid = strListArgTypesOids[i];
        	QString strQuery("SELECT typname FROM pg_catalog.pg_type typ WHERE oid=" + strArgTypeOid);
        	//QString strQuery("SELECT pg_catalog.format_type(" + strArgTypeOid + ", NULL)");
        	
        	try
        	{
        		pqxx::result pqxxResult = pqxxXaction.Exec(strQuery.latin1());
	
				if(pqxxResult.size() != 1)
				{
					kdError() << k_funcinfo "Expect one row in result !" <<  endl;
					continue;
				}
				
				strArgTypeName = pqxxResult[0][0].c_str();
			}
			catch (const std::exception &e)
			{
				kdError() << e.what() << endl;
				KPGSqlDialog dlg(0, strQuery, e.what());
				dlg.exec();
				return;
			} 
        }
        
        
        m_listArguments.append(KPGFunctionArgument(i + 1, strName, strMode, strArgTypeName));
    }
}

// Return name with arguments
const QString KPGFunction::nameWithArguments() const
{
	QString strFunctionName = text(0);
	
    strFunctionName.append("(");
	bool bFirst = true;
	for(KPGFunction::ListArguments::const_iterator cit = m_listArguments.begin(); cit != m_listArguments.end(); ++cit)
	{ 
		if(bFirst)
			bFirst = false;
		else
			strFunctionName.append(", ");
		
		if((*cit).mode() == "i") strFunctionName.append("IN ");
		else if((*cit).mode() == "o") strFunctionName.append("OUT ");
		else if((*cit).mode() == "b") strFunctionName.append("INOUT ");
		
		if((*cit).name().length() > 0)
		{
			strFunctionName.append((*cit).name());
			strFunctionName.append(" ");
		}
		
		strFunctionName.append((*cit).typName());
	}
	strFunctionName.append(")");
	
	return strFunctionName;
}
    
void KPGFunction::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
		
	// Display function info
	KPGFunctionPropPage *pPropertyPage = static_cast <KPGFunctionPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  	QHeader* hHeader = pTableProperties->horizontalHeader();

  	hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  	pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Function name"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  	pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the function"));
		
	//--- Is Aggregate
	pTableProperties->setText(iRow, 0, i18n("Is Aggregate"));
  	pTableProperties->setText(iRow, 1, (m_bIsAgg) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if function is an aggregate function)"));
	
	//--- Is Security definer
	pTableProperties->setText(iRow, 0, i18n("Is Security definer"));
  	pTableProperties->setText(iRow, 1, (m_bIsSecdef) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("Function is a security definer (i.e., a \"setuid\" function"));
			
	//--- Is Strict
	pTableProperties->setText(iRow, 0, i18n("Is Strict"));
  	pTableProperties->setText(iRow, 1, (m_bIsStrict) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("Function returns null if any call argument is null. In that case the function won't actually be called at all. Functions that are not \"strict\" must be prepared to handle null inputs."));
	
	//--- Return Set
	pTableProperties->setText(iRow, 0, i18n("Return Set"));
  	pTableProperties->setText(iRow, 1, (m_bRetSet) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("Function returns a set (i.e., multiple values of the specified data type)"));
	
	//--- Volatile
	pTableProperties->setText(iRow, 0, i18n("Volatile"));
  	pTableProperties->setText(iRow, 1, m_strVolatile);
	pTableProperties->setText(iRow++, 2, i18n("Tells whether the function's result depends only on its input arguments, or is affected by outside factors. It is i for \"immutable\" functions, which always deliver the same result for the same inputs. It is s for \"stable\" functions, whose results (for fixed inputs) do not change within a scan. It is v for \"volatile\" functions, whose results may change at any time. (Use v also for functions with side-effects, so that calls to them cannot get optimized away.) "));
	
	//--- Number of argument
	pTableProperties->setText(iRow, 0, i18n("Number of arguments"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_iNArgs));
	pTableProperties->setText(iRow++, 2, i18n("Number of arguments"));
	
	//--- Bin
	pTableProperties->setText(iRow, 0, i18n("Bin"));
  	pTableProperties->setText(iRow, 1, m_strBin);
	pTableProperties->setText(iRow++, 2, i18n("Additional information about how to invoke the function. Again, the interpretation is language-specific."));
	
	//--- Type name
	pTableProperties->setText(iRow, 0, i18n("Type name"));
  	KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Type name" );
	pLabel->setValues(m_oidReturnType, m_strReturnTypName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Return data type name"));
	
	//--- Language name
	pTableProperties->setText(iRow, 0, i18n("Language name"));
  	pTableProperties->setText(iRow, 1, m_strLanName);
	pTableProperties->setText(iRow++, 2, i18n("Name of the language (to be specified when creating a function)"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  	pTableProperties->setText(iRow, 1, m_strFuncOwner);
	pTableProperties->setText(iRow++, 2, i18n("Function owner"));
	
	//--- Namespace
	/*pTableProperties->setText(iRow, 0, i18n("Namespace"));
	pTableProperties->setText(iRow, 1, m_strNameSpace);
	pTableProperties->setText(iRow++, 2, i18n("Name of the namespace"));
	*/
	
	for(int nCol = 0; nCol < 3; nCol++)
	{
		pTableProperties->adjustColumn(nCol);
	}
	
	pPropertyPage->m_pLineEditArgTypes->setText(m_strArgTypes);
	pPropertyPage->m_pTextEditSource->setText(m_strSrc);

	//--- Access privileges
	pPropertyPage->setACL(m_strACL);
	
	//--- Arguments list
	pPropertyPage->setArgumentsList(m_listArguments);
}

// Refresh only function info, without childs objects
bool KPGFunction::refreshItem()
{
	// Get pointer to server for version info
	KPGDatabase *pDatabase = static_cast <KPGDatabase *> (parent()->parent()->parent());
	KPGServer *pServer = static_cast <KPGServer *> (pDatabase->parent());
	work pqxxXaction(* pDatabase->connection(), "refreshItem"); 
	
	// obtain function info
	QString strQuery("SELECT pr.oid, pr.proname, description, pr.proisagg, pr.prosecdef, pr.proisstrict, proretset, pr.provolatile, pr.pronargs, pr.probin, pr.proacl, prorettype, \
typ.typname, lanname, pg_catalog.pg_get_userbyid(proowner) AS funcowner, pr.proargtypes, pg_catalog.oidvectortypes(pr.proargtypes) AS arguments, pr.prosrc");
	
	if((pServer->versionMajor() >= 8) && (pServer->versionMiddle() >= 1))
    {
  		strQuery.append(", pr.proallargtypes, pr.proargmodes, pr.proargnames ");
  	}
  	else
  		strQuery.append(" ");
	
	strQuery.append("FROM pg_catalog.pg_proc pr ");
	strQuery.append("JOIN pg_catalog.pg_type typ ON typ.oid=prorettype ");
	strQuery.append("JOIN pg_catalog.pg_language LNG ON LNG.oid=prolang ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON des.objoid=pr.oid ");
	
	strQuery.append("WHERE pr.oid = " + QString("%1").arg(m_oid));
		
	try
	{
		pqxx::result pqxxResult = pqxxXaction.Exec(strQuery.latin1());
	
		if(pqxxResult.size() != 1)
		{
			kdError() << k_funcinfo "Expect one row in result !" <<  endl;
			return false;
		}
				
		bool bIsAgg;
		bool bIsSecdef;
		bool bIsStrict;
		bool bRetSet;
		int iNArgs;
		pqxx::oid oidReturnType = 0;
	
		pqxxResult[0]["proisagg"].to(bIsAgg);
		pqxxResult[0]["prosecdef"].to(bIsSecdef);
		pqxxResult[0]["proisstrict"].to(bIsStrict);
		pqxxResult[0]["proretset"].to(bRetSet);
		pqxxResult[0]["pronargs"].to(iNArgs);
		pqxxResult[0]["prorettype"].to(oidReturnType);
				
		setProperties(
		pqxxResult[0]["description"].c_str(), 
		bIsAgg,
		bIsSecdef,
		bIsStrict,
		bRetSet,
		pqxxResult[0]["provolatile"].c_str(), 
		iNArgs,
		pqxxResult[0]["probin"].c_str(), 
		pqxxResult[0]["proacl"].c_str(), 
		pqxxResult[0]["typname"].c_str(),
		oidReturnType,
		pqxxResult[0]["lanname"].c_str(), 
		pqxxResult[0]["funcowner"].c_str()
		);
		
		setArgTypes(pqxxResult[0]["arguments"].c_str());
		setSrc(pqxxResult[0]["prosrc"].c_str());
			
		if((pServer->versionMajor() >= 8) && (pServer->versionMiddle() >= 1))
    	{	
			setListArguments(pqxxResult[0]["proargtypes"].c_str(),
			    pqxxResult[0]["proallargtypes"].c_str(),
				pqxxResult[0]["proargmodes"].c_str(),
				pqxxResult[0]["proargnames"].c_str(),
				pqxxXaction);
		}
		else
		{
			setListArguments(pqxxResult[0]["proargtypes"].c_str(),
			    QString::null,
				QString::null,
				QString::null,
				pqxxXaction);
		}
	
	}
	catch (const std::exception &e)
	{
		kdError() << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return false;
	} 
	
	return true;
}

