/***************************************************************************
                          kpgfunctionsfolder  -  description
                             -------------------
    begin                : Ne no 1 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "kpgfunctionsfolder.h"

#include <qtable.h>

#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgconnection.h"
#include "kpgserver.h"
#include "kpgschema.h"
#include "kpgfunction.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "../kpgsqldialog.h"

KPGFunctionsFolder::KPGFunctionsFolder(KPGSchema *parent, pqxx::oid oidNameSpace)
 : KPGItemsFolder(parent, i18n("Functions"))
{
	setPixmap(0, *m_pIconFolderRed);
	m_oidNameSpace = oidNameSpace;
}


KPGFunctionsFolder::~KPGFunctionsFolder()
{
}

bool KPGFunctionsFolder::refresh(pqxx::work &pqxxXaction)
{
    // delete all child items
  	while (QListViewItem * pItem = firstChild())
    	delete pItem;

	// Get pointer to server for version info       schema ->database->server 
	KPGServer *pServer = static_cast <KPGServer *> (parent()->parent()->parent());

	// obtain list of functions
	QString strQuery("SELECT pr.oid, pr.proname, description, pr.proisagg, pr.prosecdef, pr.proisstrict, proretset, pr.provolatile, pr.pronargs, pr.probin, pr.proacl, prorettype, \
typ.typname, lanname, pg_catalog.pg_get_userbyid(proowner) as funcowner, pr.proargtypes, pg_catalog.oidvectortypes(pr.proargtypes) AS arguments, pr.prosrc");
  
  	if((pServer->versionMajor() >= 8) && (pServer->versionMiddle() >= 1))
    {
  		strQuery.append(", pr.proallargtypes, pr.proargmodes, pr.proargnames ");
  	}
  	else
  		strQuery.append(" ");
  	
	strQuery.append("FROM pg_catalog.pg_proc pr ");
	strQuery.append("JOIN pg_catalog.pg_type typ ON typ.oid=prorettype ");
	strQuery.append("JOIN pg_catalog.pg_language LNG ON LNG.oid=prolang ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON des.objoid=pr.oid ");
	strQuery.append("WHERE pronamespace = " + QString("%1").arg(m_oidNameSpace) + " ");
	strQuery.append("ORDER BY proname");
			
  try
  {
    	m_pqxxResultFunctions = pqxxXaction.Exec(strQuery.latin1());

		KPGFunction *pFunction = 0;
		
		for (result::size_type i = 0; i != m_pqxxResultFunctions.size(); ++i)
		{
				pqxx::oid oid;
				m_pqxxResultFunctions[i][0].to(oid);
			
				if(pFunction == 0)
					pFunction = new KPGFunction(this, m_pqxxResultFunctions[i]["proname"].c_str(), oid);
				else
					pFunction = new KPGFunction(this, pFunction, m_pqxxResultFunctions[i]["proname"].c_str(), oid);
				
				bool bIsAgg;
				bool bIsSecdef;
				bool bIsStrict;
				bool bRetSet;
				int iNArgs;
				pqxx::oid oidReturnType = 0;
	
				m_pqxxResultFunctions[i]["proisagg"].to(bIsAgg);
				m_pqxxResultFunctions[i]["prosecdef"].to(bIsSecdef);
				m_pqxxResultFunctions[i]["proisstrict"].to(bIsStrict);
				m_pqxxResultFunctions[i]["proretset"].to(bRetSet);
				m_pqxxResultFunctions[i]["pronargs"].to(iNArgs);
				m_pqxxResultFunctions[i]["prorettype"].to(oidReturnType);
							
				pFunction->setProperties(
					m_pqxxResultFunctions[i]["description"].c_str(), 
					bIsAgg,
					bIsSecdef,
					bIsStrict,
					bRetSet,
					m_pqxxResultFunctions[i]["provolatile"].c_str(), 
					iNArgs,
					m_pqxxResultFunctions[i]["probin"].c_str(), 
					m_pqxxResultFunctions[i]["proacl"].c_str(), 
					m_pqxxResultFunctions[i]["typname"].c_str(),
					oidReturnType,
					m_pqxxResultFunctions[i]["lanname"].c_str(), 
					m_pqxxResultFunctions[i]["funcowner"].c_str()
					);
				
				pFunction->setArgTypes(m_pqxxResultFunctions[i]["arguments"].c_str());
				pFunction->setSrc(m_pqxxResultFunctions[i]["prosrc"].c_str());
					
				if((pServer->versionMajor() >= 8) && (pServer->versionMiddle() >= 1))
    			{	
					pFunction->setListArguments(m_pqxxResultFunctions[i]["proargtypes"].c_str(),
					    m_pqxxResultFunctions[i]["proallargtypes"].c_str(),
						m_pqxxResultFunctions[i]["proargmodes"].c_str(),
						m_pqxxResultFunctions[i]["proargnames"].c_str(),
						pqxxXaction);
				}
				else
				{
					pFunction->setListArguments(m_pqxxResultFunctions[i]["proargtypes"].c_str(),
						QString::null,
						QString::null,
						QString::null,
						pqxxXaction);
				}
		}
	}
	catch (const std::exception &e)
	{
		kdError() << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return false;
	} 
	
	return true;
}

void KPGFunctionsFolder::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  	pPropertyPage->setLabelText(i18n("Functions &list:"));
	
	//----------------------------------------------------------------------------
	//Display functions list
		
	QTable *pTableFunctions = pPropertyPage->m_pTableItems;
  
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultFunctions, pTableFunctions, true, 13);
	
	// field names
	QHeader* hHeader = pTableFunctions->horizontalHeader();

  	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Is Aggregate"));
  	hHeader->setLabel(nCol++, i18n("Is Security definer"));
	hHeader->setLabel(nCol++, i18n("Is Strict"));
	hHeader->setLabel(nCol++, i18n("Return Set"));
	hHeader->setLabel(nCol++, i18n("Volatile"));
	hHeader->setLabel(nCol++, i18n("Number of arguments"));
	hHeader->setLabel(nCol++, i18n("Bin"));
	hHeader->setLabel(nCol++, i18n("ACL"));
	hHeader->setLabel(nCol++, i18n("Type name"));
	hHeader->setLabel(nCol++, i18n("Language name"));
	hHeader->setLabel(nCol++, i18n("Owner"));

    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        pTableFunctions->adjustColumn(nCol);
    }
}

void KPGFunctionsFolder::fillListOfObjectOidsWithChildItems(KPGOidNameList & listOfObjectOids)
{
    QListViewItem * pItem = firstChild();
    while(pItem)
    {
        KPGTreeItem *pTreeItem = static_cast <KPGTreeItem *> (pItem);
        
        if(pTreeItem->type() != nodeFunction)
        {
            kdError() << "KPGFunctionsFolder ust have KPGFunction childs !" << endl;
            return;
        }
        
        KPGFunction *pFunction = static_cast <KPGFunction *> (pItem);
        
        listOfObjectOids.append(KPGOidName(pFunction->oid(), pFunction->text(0) + pFunction->argumentTypes(), pFunction->type(), pFunction->pixmap(0)));
        
        pItem = pItem->nextSibling();
    }
}

