//
// C++ Implementation: kpggroupsfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpggroupsfolder.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "kpgconnection.h"
#include "kpgserver.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "kpggroup.h"
#include "../kpgsqldialog.h"

KPGGroupsFolder::KPGGroupsFolder(KPGServer *parent)
 : KPGItemsFolder(parent, i18n("Groups"))
{
	setPixmap(0, *m_pIconFolderGrey);
}


KPGGroupsFolder::~KPGGroupsFolder()
{
}

bool KPGGroupsFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items (databases)
  while (QListViewItem * pItem = firstChild())
    delete pItem;

	// obtain list of tables
  QString strQuery("SELECT groname, grosysid FROM pg_catalog.pg_group ORDER BY groname;");
			
  try
  {
    m_pqxxResultGroups = pqxxXaction.Exec(strQuery.latin1());

		KPGGroup *pgGroup = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultGroups.size(); ++i)
    {
			if(pgGroup == 0)
				pgGroup = new KPGGroup(this, m_pqxxResultGroups[i]["groname"].c_str());
			else
				pgGroup = new KPGGroup(this, pgGroup, m_pqxxResultGroups[i]["groname"].c_str());
			
			int iSysId; 

			m_pqxxResultGroups[i]["grosysid"].to(iSysId);
		
			pgGroup->setProperties(iSysId);
			
			//-- Get users list
			strQuery = QString("SELECT grolist FROM pg_catalog.pg_group WHERE grosysid=%1;").arg(iSysId);
			pqxx::result pqxxResultUsersList = pqxxXaction.exec(strQuery.latin1());
		
			if(pqxxResultUsersList.size() > 0)
    	{ 
			  //fieldstream fs(pqxxResultUsersList[0]["grolist"]);
				QString strUsrList(pqxxResultUsersList[0]["grolist"].c_str());
				
				if(strUsrList.length() > 0)
				{
					strUsrList.remove('{');
					strUsrList.remove('}');
													
					strQuery = QString("SELECT usename FROM pg_catalog.pg_user WHERE usesysid IN (%1);").arg(strUsrList);
					pqxx::result pqxxResultUser = pqxxXaction.exec(strQuery.latin1());
						
					for(result::size_type j = 0; j != pqxxResultUser.size(); ++j)
					{
						pgGroup->getListOfUsers().append(pqxxResultUser[j]["usename"].c_str());
					}
				}
			} 
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}
void KPGGroupsFolder::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  	pPropertyPage->setLabelText(i18n("Groups &list:"));
	
	//----------------------------------------------------------------------------
	//Display views list
		
	// Fill output area with query result
	int nRows = m_pqxxResultGroups.size();
	int nCols = m_pqxxResultGroups.columns();

	QTable *pTableGroups = pPropertyPage->m_pTableItems;
	
	pTableGroups->setNumRows(nRows);
	pTableGroups->setNumCols(nCols);
	
	// field names
	QHeader* hHeader = pTableGroups->horizontalHeader();
	
	int nCol = 0;
		hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("System ID"));
		
	// data
	for(int nRow = 0; nRow < nRows; nRow++)
	{
		for(int nCol = 0; nCol < nCols; nCol++)
		{
		QString strValue(m_pqxxResultGroups[nRow][nCol].c_str());
		pTableGroups->setText(nRow, nCol, strValue);
		}
	}
	
	for(int nCol = 0; nCol < nCols; nCol++)
	{
		pTableGroups->adjustColumn(nCol);
	}
}


