//
// C++ Implementation: kpgoperator
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgoperator.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktextedit.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpgoperatorsfolder.h"
#include "../PropertyPages/kpgoperatorproppage.h"


KPGOperator::KPGOperator(KPGOperatorsFolder *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
  setPixmap(0, *m_pIconOperator);
}

KPGOperator::KPGOperator(KPGOperatorsFolder *parent, KPGOperator *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconOperator);
}


KPGOperator::~KPGOperator()
{
}

void KPGOperator::setProperties(
		const QString strDescription,
		const QString strOwner,
		const QString strKind,
		bool bCanHash,
		pqxx::oid oidLeftType,
		const QString strLeftType,
		pqxx::oid oidRightType,
		const QString strRightType,
		pqxx::oid oidResultType,
		const QString strResultType,
		pqxx::oid oidComutatorOpr,
		const QString strComutatorOprName,
		pqxx::oid oidNegatorOpr,
		const QString strNegatorOprName,
		pqxx::oid oidLeftSortOpr,
		const QString strLeftSortOprName,
		pqxx::oid oidRightSortOpr,
		const QString strRightSortOprName,
		pqxx::oid oidLessThanOpr,
		const QString strLessThanOprName,
		pqxx::oid oidGreatherThanOpr,
		const QString strGreatherThanOprName,
		pqxx::oid oidOperatorFunc,
		const QString strOperatorFunc,
		pqxx::oid oidRestrictionFunc,
		const QString strRestrictionFunc,
		pqxx::oid oidJoinFunc,
		const QString strJoinFunc
	)
{
	m_strDescription = strDescription;
	m_strOwner = strOwner;
	m_strKind = strKind;
	m_bCanHash = bCanHash;
	m_oidLeftType = oidLeftType;
	m_strLeftType = strLeftType;
	m_oidRightType = oidRightType;
	m_strRightType = strRightType;
	m_oidResultType = oidResultType;
	m_strResultType = strResultType;
	m_oidComutatorOpr = oidComutatorOpr;
	m_strComutatorOprName = strComutatorOprName;
	m_oidNegatorOpr = oidNegatorOpr;
	m_strNegatorOprName = strNegatorOprName;
	m_oidLeftSortOpr = oidLeftSortOpr;
 	m_strLeftSortOprName = strLeftSortOprName;
	m_oidRightSortOpr = oidRightSortOpr;
	m_strRightSortOprName = strRightSortOprName;
	m_oidLessThanOpr = oidLessThanOpr;
  m_strLessThanOprName = strLessThanOprName;
	m_oidGreatherThanOpr = oidGreatherThanOpr;
	m_strGreatherThanOprName = strGreatherThanOprName;
	m_oidOperatorFunc = oidOperatorFunc;
	m_strOperatorFunc = strOperatorFunc;
	m_oidRestrictionFunc = oidRestrictionFunc;
	m_strRestrictionFunc = strRestrictionFunc;
	m_oidJoinFunc = oidJoinFunc;
	m_strJoinFunc = strJoinFunc;
}

void KPGOperator::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	// Display operator info
	KPGOperatorPropPage *pPropertyPage = static_cast <KPGOperatorPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Operator name"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the operator"));
	
	//--- Kind
	pTableProperties->setText(iRow, 0, i18n("Kind"));
  pTableProperties->setText(iRow, 1, m_strKind);
	pTableProperties->setText(iRow++, 2, i18n("b = infix (\"both\"), l = prefix (\"left\"), r = postfix (\"right\") "));
	
	//--- Can Hash
	pTableProperties->setText(iRow, 0, i18n("Can Hash"));
  pTableProperties->setText(iRow, 1, (m_bCanHash) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("This operator supports hash joins"));
	
	//--- Left Type name
	pTableProperties->setText(iRow, 0, i18n("Left type"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Left type" );
	pLabel->setValues(m_oidLeftType, m_strLeftType);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Type of the left operand"));
	
	//--- Right Type name
	pTableProperties->setText(iRow, 0, i18n("Right type"));
  pLabel = new KPGLinkLabel( pTableProperties, "Right type" );
	pLabel->setValues(m_oidRightType, m_strRightType);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Type of the right operand"));
	
	//--- Return Type name
	pTableProperties->setText(iRow, 0, i18n("Result type"));
  pLabel = new KPGLinkLabel( pTableProperties, "Result type" );
	pLabel->setValues(m_oidResultType, m_strResultType);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Type of the result"));
	
	//--- Comutator
	pTableProperties->setText(iRow, 0, i18n("Comutator"));
  pLabel = new KPGLinkLabel( pTableProperties, "Comutator" );
	pLabel->setValues(m_oidComutatorOpr, m_strComutatorOprName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Commutator of this operator, if any"));
	
	//--- Negator
	pTableProperties->setText(iRow, 0, i18n("Negator"));
  pLabel = new KPGLinkLabel( pTableProperties, "Negator" );
	pLabel->setValues(m_oidNegatorOpr, m_strNegatorOprName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Negator of this operator, if any"));
		
	//--- Left-hand Ooperator
	pTableProperties->setText(iRow, 0, i18n("Left-hand operator"));
  pLabel = new KPGLinkLabel( pTableProperties, "Left-hand operator" );
	pLabel->setValues(m_oidLeftSortOpr, m_strLeftSortOprName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("If this operator supports merge joins, the operator that sorts the type of the left-hand operand (L<L) "));
	
	//--- Right-hand Ooperator
	pTableProperties->setText(iRow, 0, i18n("Right-hand operator"));
  pLabel = new KPGLinkLabel( pTableProperties, "Right-hand operator" );
	pLabel->setValues(m_oidRightSortOpr, m_strRightSortOprName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("If this operator supports merge joins, the operator that sorts the type of the right-hand operand (R<R) "));
	
	//--- Less-than operator
	pTableProperties->setText(iRow, 0, i18n("Less-than operator"));
  pLabel = new KPGLinkLabel( pTableProperties, "Less-than operator" );
	pLabel->setValues(m_oidLessThanOpr, m_strLessThanOprName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);	
	pTableProperties->setText(iRow++, 2, i18n("If this operator supports merge joins, the less-than operator that compares the left and right operand types (L<R) "));
	
	//--- Greater-than Operator
	pTableProperties->setText(iRow, 0, i18n("Greater-than operator"));
  pLabel = new KPGLinkLabel( pTableProperties, "Greater-than operator" );
	pLabel->setValues(m_oidGreatherThanOpr, m_strGreatherThanOprName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("If this operator supports merge joins, the greater-than operator that compares the left and right operand types (L>R)"));
	
	//--- Function
	pTableProperties->setText(iRow, 0, i18n("Function"));
  pLabel = new KPGLinkLabel( pTableProperties, "Function" );
	pLabel->setValues(m_oidOperatorFunc, m_strOperatorFunc);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Function that implements this operator"));
		
	//--- Restriction func.
	pTableProperties->setText(iRow, 0, i18n("Restriction func."));
  pLabel = new KPGLinkLabel( pTableProperties, "Restriction func." );
	pLabel->setValues(m_oidRestrictionFunc, m_strRestrictionFunc);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Restriction selectivity estimation function for this operator"));
	
	//--- Join func.
	pTableProperties->setText(iRow, 0, i18n("Join function"));
  pLabel = new KPGLinkLabel( pTableProperties, "Join func." );
	pLabel->setValues(m_oidJoinFunc, m_strJoinFunc);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Join selectivity estimation function for this operator"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Operator owner"));
	
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}
