//
// C++ Implementation: kpgschema
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgschema.h"

// include files for KDE
#include <kiconloader.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kcursor.h>

#include <qtable.h>

#include "kpgdatabase.h"
#include "kpgserver.h"
#include "../PropertyPages/kpgschemaproppage.h"
#include "../kpglinklabel.h"
#include "../kpgsqldialog.h"

#include "kpgconnection.h"
#include "kpgtablesfolder.h"
#include "kpgviewsfolder.h"
#include "kpgsequencesfolder.h"
#include "kpgfunctionsfolder.h"
#include "kpgoperatorsfolder.h"
#include "kpgoperatorclassesfolder.h"
#include "kpgaggregatesfolder.h"
#include "kpgdomainsfolder.h"
#include "kpgtypesfolder.h"
#include "kpgconversionsfolder.h"

KPGSchema::KPGSchema(KPGDatabase *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
  m_bIsLoaded = false;
  setPixmap(0, *m_pIconSchemaUnl);
  
  if((name.left(3) == "pg_") || (name == "information_schema"))
    m_bIsSystemObject = true;
  else
    m_bIsSystemObject = false;
  
    m_pFolderViews = new KPGViewsFolder(this, _oid);
    m_pFolderSequences = new KPGSequencesFolder(this, _oid);
    m_pFolderTypes = new KPGTypesFolder(this, _oid);
    m_pFolderTables = new KPGTablesFolder(this, _oid);
    m_pFolderOperatorClasses = new KPGOperatorClassesFolder(this, _oid);
	m_pFolderOperators = new KPGOperatorsFolder(this, _oid);
    m_pFolderFunctions = new KPGFunctionsFolder(this, _oid);
    m_pFolderDomains = new KPGDomainsFolder(this, _oid);
	m_pFolderConversions = new KPGConversionsFolder(this, _oid);
	m_pFolderAggregates = new KPGAggregatesFolder(this, _oid);
  
	m_bPageAggregatesDisplaied = false;
	m_bPageConversionsDisplaied = false;
	m_bPageDomainsDisplaied = false;
	m_bPageFunctionsDisplaied = false;
	m_bPageOperatorClassesDisplaied = false;
	m_bPageOperatorsDisplaied = false;
	m_bPageSequencesDisplaied = false;
    m_bPageTablesDisplaied = false;
    m_bPageTypesDisplaied = false;
	m_bPageViewsDisplaied = false;
}

KPGSchema::KPGSchema(KPGDatabase *parent, KPGSchema *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
  m_bIsLoaded = false;
  setPixmap(0, *m_pIconSchemaUnl);
  
  if((name.left(3) == "pg_") || (name == "information_schema"))
	 	m_bIsSystemObject = true;
  else
	  m_bIsSystemObject = false;
  	
	m_pFolderViews = new KPGViewsFolder(this, _oid);
    m_pFolderSequences = new KPGSequencesFolder(this, _oid);
    m_pFolderTypes = new KPGTypesFolder(this, _oid);
    m_pFolderTables = new KPGTablesFolder(this, _oid);
    m_pFolderOperatorClasses = new KPGOperatorClassesFolder(this, _oid);
    m_pFolderOperators = new KPGOperatorsFolder(this, _oid);
    m_pFolderFunctions = new KPGFunctionsFolder(this, _oid);
    m_pFolderDomains = new KPGDomainsFolder(this, _oid);
    m_pFolderConversions = new KPGConversionsFolder(this, _oid);
    m_pFolderAggregates = new KPGAggregatesFolder(this, _oid);
  
	m_bPageAggregatesDisplaied = false;
	m_bPageConversionsDisplaied = false;
	m_bPageDomainsDisplaied = false;
	m_bPageFunctionsDisplaied = false;
	m_bPageOperatorClassesDisplaied = false;
	m_bPageOperatorsDisplaied = false;
	m_bPageSequencesDisplaied = false;
    m_bPageTablesDisplaied = false;
    m_bPageTypesDisplaied = false;
	m_bPageViewsDisplaied = false;
}

KPGSchema::~KPGSchema()
{
}

void KPGSchema::activate()
{
  if(!m_bIsLoaded)
  { 
    listView()->setCursor(KCursor::waitCursor());
    refresh();
    listView()->setCursor(KCursor::arrowCursor());
    setOpen(true);
  }
}

bool KPGSchema::refresh()
{
  KPGConnection *pDbConnection = connection();
	pqxx::work pqxxXaction(*pDbConnection, "SchemaRefresh");
	
  return refresh(pqxxXaction);
}

bool KPGSchema::refresh(pqxx::work &pqxxXaction)
{	
	if(!m_pFolderTables->refresh(pqxxXaction)) return false;
	if(!m_pFolderViews->refresh(pqxxXaction)) return false;
	if(!m_pFolderFunctions->refresh(pqxxXaction)) return false;
	if(!m_pFolderSequences->refresh(pqxxXaction)) return false;
	if(!m_pFolderOperators->refresh(pqxxXaction)) return false;
	if(!m_pFolderOperatorClasses->refresh(pqxxXaction)) return false;
	if(!m_pFolderAggregates->refresh(pqxxXaction)) return false;
	if(!m_pFolderDomains->refresh(pqxxXaction)) return false;
	if(!m_pFolderTypes->refresh(pqxxXaction)) return false;
	if(!m_pFolderConversions->refresh(pqxxXaction)) return false;
				
    m_bIsLoaded = true;
  
    if(m_bIsSystemObject)
        setPixmap(0, *m_pIconSchemaSys);
    else
        setPixmap(0, *m_pIconSchemaUsr);
    
	return true;
}

// Refresh only schema info, without childs objects
bool KPGSchema::refreshItem()
{
    // Get pointer to server for version info
    KPGDatabase *pDatabase = static_cast <KPGDatabase *> (parent());
    //KPGServer *pServer = static_cast <KPGServer *> (pDatabase->parent());
    
    // obtain schema
    QString strQuery("SELECT nsp.nspname, description, nspacl, has_schema_privilege(nsp.oid, 'CREATE') as cancreate, pg_get_userbyid(nspowner) AS owner ");
    
    // Note: Information about tablespace was removed from schema between PostgreSQL 8 b4 and 8.0 final
    //if(pServer->versionMajor() >= 8)
    //  strQuery.append(", nsp.nsptablespace, ts.spcname ");
    
    strQuery.append("FROM pg_catalog.pg_namespace nsp ");
    strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON des.objoid=nsp.oid ");
    
    //if(pServer->versionMajor() >= 8)
    //  strQuery.append("LEFT JOIN pg_tablespace ts ON nsp.nsptablespace=ts.oid ");
        
    strQuery.append("WHERE nsp.oid = " + QString("%1").arg(m_oid));
        
    try
    {
        work pqxxXaction(*pDatabase->connection(), "refreshItem");   
        
        pqxx::result pqxxResultSchemas = pqxxXaction.Exec(strQuery.latin1());
    
        if(pqxxResultSchemas.size() != 1)
        {
        kdError() << k_funcinfo "Expect one row in result !" <<  endl;
        return false;
        }
        
        bool bCanCreate;
        pqxxResultSchemas[0]["cancreate"].to(bCanCreate);
        
        // Note: Information about tablespace was removed from schema between PostgreSQL 8 b4 and 8.0 final
        /*
        if(pServer->versionMajor() <= 7)
        {*/
        setProperties(
            pqxxResultSchemas[0]["owner"].c_str(),
            bCanCreate, 
            pqxxResultSchemas[0]["nspacl"].c_str(),
            pqxxResultSchemas[0]["description"].c_str()
            );
        /*}
        else
        {
        pqxx::oid oidTablespace;
        pqxxResultSchemas[0]["nsptablespace"].to(oidTablespace);
            
        setProperties(
            pqxxResultSchemas[0]["owner"].c_str(),
            bCanCreate, 
            pqxxResultSchemas[0]["nspacl"].c_str(),
            pqxxResultSchemas[0]["description"].c_str(),
            oidTablespace,
            pqxxResultSchemas[0]["spcname"].c_str()
            );
        }*/
        
    }
    catch (const std::exception &e)
    {
        kdError() << e.what() << endl;
        KPGSqlDialog dlg(0, strQuery, e.what());
        dlg.exec();
        return false;
    } 
        
    return true;
}

void KPGSchema::displayTabPage(KPGSchemaPropPage *pPropertyPage, QWidget *pPage)
{
	pPropertyPage->setCursor(KCursor::waitCursor());	
		
	if((pPage == pPropertyPage->tabAggregates) && (!m_bPageAggregatesDisplaied))
		displayAggregates(pPropertyPage);
		
	if((pPage == pPropertyPage->tabConversions) && (!m_bPageConversionsDisplaied))
		displayConversions(pPropertyPage);
	
	if((pPage == pPropertyPage->tabDomains) && (!m_bPageDomainsDisplaied))
		displayDomains(pPropertyPage);
			
	if((pPage == pPropertyPage->tabFunctions) && (!m_bPageFunctionsDisplaied))
		displayFunctions(pPropertyPage);
		
	if((pPage == pPropertyPage->tabOperatorClasses) && (!m_bPageOperatorClassesDisplaied))
	  displayOperatorClasses(pPropertyPage);
		
	if((pPage == pPropertyPage->tabOperators) && (!m_bPageOperatorsDisplaied))
		displayOperators(pPropertyPage);
		
	if((pPage == pPropertyPage->tabSequences) && (!m_bPageSequencesDisplaied))
	  displaySequences(pPropertyPage);
		
	if((pPage == pPropertyPage->tabTables) && (!m_bPageTablesDisplaied))
		displayTables(pPropertyPage);
			
	if((pPage == pPropertyPage->tabTypes) && (!m_bPageTypesDisplaied))
		displayTypes(pPropertyPage);

	if((pPage == pPropertyPage->tabViews) && (!m_bPageViewsDisplaied))
		displayViews(pPropertyPage);
				
	pPropertyPage->setCursor(KCursor::arrowCursor());
}

// For PostgreSQL 7.x or older 
void KPGSchema::setProperties(
	const QString strOwner, 
	bool bCanCreate,
	const QString strACL,
	const QString strDescription
	)
{
	m_strOwner = strOwner;
	m_bCanCreate = bCanCreate;
	m_strACL = strACL;
	m_strDescription = strDescription;
	//m_oidTablespace = 0;
}

// Note: Information about tablespace was removed from schema between PostgreSQL 8 b4 and 8.0 final
// For PostgreSQL 8 or newer 
/*void KPGSchema::setProperties(
	const QString strOwner, 
	bool bCanCreate,
	const QString strACL,
	const QString strDescription,
	pqxx::oid oidTablespace,   
  QString strTablespace
	)
{
	m_strOwner = strOwner;
	m_bCanCreate = bCanCreate;
	m_strACL = strACL;
	m_strDescription = strDescription;
	m_oidTablespace = oidTablespace;
	m_strTablespace = strTablespace;
}*/

void KPGSchema::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	// Display schema properties
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
		
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  	QHeader* hHeader = pTableProperties->horizontalHeader();

  	hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
    pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Name of the schema"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  	pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the schema"));
	
	//--- ACL
	pPropertyPage->setACL(m_strACL);
	
	//--- Can Create
	pTableProperties->setText(iRow, 0, i18n("Can Create"));
  	pTableProperties->setText(iRow, 1, (m_bCanCreate) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("Does current user have CREATE privilege for schema ? "));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  	pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the schema"));
	
	//--- Tablespace
    // Note: Information about tablespace was removed from schema between PostgreSQL 8 b4 and 8.0 final
	/*pTableProperties->setText(iRow, 0, i18n("Tablespace"));
    KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Tablespace" );
    pLabel->setValues(m_oidTablespace, m_strTablespace);
    pTableProperties->setCellWidget(iRow, 1, pLabel);
    pTableProperties->setText(iRow++, 2, i18n("Tablespace name"));*/
			
	for(int nCol = 0; nCol < 3; nCol++)
    {
        pTableProperties->adjustColumn(nCol);
    }
	
	m_bPageAggregatesDisplaied = false;
	m_bPageConversionsDisplaied = false;
	m_bPageDomainsDisplaied = false;
	m_bPageFunctionsDisplaied = false;
	m_bPageOperatorClassesDisplaied = false;
	m_bPageOperatorsDisplaied = false;
	m_bPageSequencesDisplaied = false;
	m_bPageTablesDisplaied = false;
	m_bPageTypesDisplaied = false;
	m_bPageViewsDisplaied = false;
}

void KPGSchema::displayAggregates(QWidget *widgetProperies)
{
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display aggregates list
	QTable *pTableAggregates = pPropertyPage->m_pTableAggregates;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderAggregates->resultAggregates(), pTableAggregates);

    // field names
    QHeader* hHeader = pTableAggregates->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Aggregate function"));
    hHeader->setLabel(nCol++, i18n("Owner"));
	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Transition function"));
    hHeader->setLabel(nCol++, i18n("Final function"));
	hHeader->setLabel(nCol++, i18n("Input type"));
	hHeader->setLabel(nCol++, i18n("Transition type"));
	hHeader->setLabel(nCol++, i18n("Return type"));
	hHeader->setLabel(nCol++, i18n("Initial value"));

    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        pTableAggregates->adjustColumn(nCol);
    }
	
	m_bPageAggregatesDisplaied = true;
}

void KPGSchema::displayConversions(QWidget *widgetProperies)
{
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display conversions list
	QTable *pTableConversions = pPropertyPage->m_pTableConversions;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderConversions->resultConversions(), pTableConversions);

    // field names
    QHeader* hHeader = pTableConversions->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
    hHeader->setLabel(nCol++, i18n("Is Default"));
    hHeader->setLabel(nCol++, i18n("Source encoding"));
    hHeader->setLabel(nCol++, i18n("Destination encoding"));
 	hHeader->setLabel(nCol++, i18n("Conversion procedure"));
	hHeader->setLabel(nCol++, i18n("Owner"));

    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        pTableConversions->adjustColumn(nCol);
    }
	
	m_bPageConversionsDisplaied = true;
}

void KPGSchema::displayDomains(QWidget *widgetProperies)
{
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display domains list
	QTable *pTableDomains = pPropertyPage->m_pTableDomains;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderDomains->resultDomains(), pTableDomains);

    // field names
    QHeader* hHeader = pTableDomains->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Base type"));
  	hHeader->setLabel(nCol++, i18n("Type length"));
	hHeader->setLabel(nCol++, i18n("Not null"));
	hHeader->setLabel(nCol++, i18n("Default value"));
	hHeader->setLabel(nCol++, i18n("Dimensions"));
	hHeader->setLabel(nCol++, i18n("Array delimiter"));
	hHeader->setLabel(nCol++, i18n("Owner"));

    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        pTableDomains->adjustColumn(nCol);
    }
	
	m_bPageDomainsDisplaied = true;
}

void KPGSchema::displayFunctions(QWidget *widgetProperies)
{
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display functions list
	QTable *pTableFunctions = pPropertyPage->m_pTableFunctions;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderFunctions->resultFunctions(), pTableFunctions, true, 13);

	// field names
	QHeader* hHeader = pTableFunctions->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Is Aggregate"));
  	hHeader->setLabel(nCol++, i18n("Is Security definer"));
	hHeader->setLabel(nCol++, i18n("Is Strict"));
  	hHeader->setLabel(nCol++, i18n("Return Set"));
  	hHeader->setLabel(nCol++, i18n("Volatile"));
  	hHeader->setLabel(nCol++, i18n("Number of arguments"));
  	hHeader->setLabel(nCol++, i18n("Bin"));
	hHeader->setLabel(nCol++, i18n("ACL"));
	hHeader->setLabel(nCol++, i18n("Type name"));
	hHeader->setLabel(nCol++, i18n("Language name"));
	hHeader->setLabel(nCol++, i18n("Owner"));

	for(int nCol = 0; nCol < nTableCols; nCol++)
	{
		pTableFunctions->adjustColumn(nCol);
	}
	
	m_bPageFunctionsDisplaied = true;
}

void KPGSchema::displayOperatorClasses(QWidget *widgetProperies)
{
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display operator classes list
	QTable *pTableOperatorClasses = pPropertyPage->m_pTableOperatorClasses;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderOperatorClasses->resultOperatorClasses(), pTableOperatorClasses);

	// field names
	QHeader* hHeader = pTableOperatorClasses->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  	hHeader->setLabel(nCol++, i18n("Is Defalt"));
	hHeader->setLabel(nCol++, i18n("Data type"));
  	hHeader->setLabel(nCol++, i18n("Key Type"));
	hHeader->setLabel(nCol++, i18n("Access method"));
	hHeader->setLabel(nCol++, i18n("Owner"));

	for(int nCol = 0; nCol < nTableCols; nCol++)
	{
		pTableOperatorClasses->adjustColumn(nCol);
	}
	
	m_bPageOperatorClassesDisplaied = true;
}

void KPGSchema::displayOperators(QWidget *widgetProperies)
{
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display operators list
	QTable *pTableOperators = pPropertyPage->m_pTableOperators;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderOperators->resultOperators(), pTableOperators);

    // field names
  	QHeader* hHeader = pTableOperators->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Kind"));
  	hHeader->setLabel(nCol++, i18n("Can Hash"));
	hHeader->setLabel(nCol++, i18n("Left Type"));
	hHeader->setLabel(nCol++, i18n("Right Type"));
	hHeader->setLabel(nCol++, i18n("Result Type"));
	hHeader->setLabel(nCol++, i18n("Comutator"));
	hHeader->setLabel(nCol++, i18n("Negator"));
	hHeader->setLabel(nCol++, i18n("Left-hand"));
	hHeader->setLabel(nCol++, i18n("Right-hand"));
	hHeader->setLabel(nCol++, i18n("Less-than"));
	hHeader->setLabel(nCol++, i18n("Greater-than"));
	hHeader->setLabel(nCol++, i18n("Function"));
	hHeader->setLabel(nCol++, i18n("Restriction func."));
	hHeader->setLabel(nCol++, i18n("Join func."));
	hHeader->setLabel(nCol++, i18n("Owner"));

	for(int nCol = 0; nTableCols < nTableCols; nCol++)
	{
		pTableOperators->adjustColumn(nCol);
	}
	
	m_bPageOperatorsDisplaied = true;
}

void KPGSchema::displaySequences(QWidget *widgetProperies)
{
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display sequences list
	QTable *pTableSequences = pPropertyPage->m_pTableSequences;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderSequences->resultSequences(), pTableSequences);

	// field names
	QHeader* hHeader = pTableSequences->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Owner"));
  	hHeader->setLabel(nCol++, i18n("ACL"));

	for(int nCol = 0; nCol < nTableCols; nCol++)
	{
		pTableSequences->adjustColumn(nCol);
	}
	
	m_bPageSequencesDisplaied = true;
}

void KPGSchema::displayTables(QWidget *widgetProperies)
{
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display tables list
	QTable *pTableTables = pPropertyPage->m_pTableTables;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderTables->resultTables(), pTableTables, true, 9);

  // field names
  	QHeader* hHeader = pTableTables->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Owner"));
  	hHeader->setLabel(nCol++, i18n("ACL"));
	hHeader->setLabel(nCol++, i18n("Has OIDs"));
	hHeader->setLabel(nCol++, i18n("Has subclass"));
	hHeader->setLabel(nCol++, i18n("Estimated rows"));
	hHeader->setLabel(nCol++, i18n("Primary key name"));
	hHeader->setLabel(nCol++, i18n("Primary key columns"));

	for(int nCol = 0; nCol < nTableCols; nCol++)
	{
		pTableTables->adjustColumn(nCol);
	}
	
	m_bPageTablesDisplaied = true;
}

void KPGSchema::displayTypes(QWidget *widgetProperies)
{
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display types list
	QTable *m_pTableTypes = pPropertyPage->m_pTableTypes;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderTypes->resultTypes(), m_pTableTypes);

    // field names
    QHeader* hHeader = m_pTableTypes->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
    hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Type"));
    hHeader->setLabel(nCol++, i18n("Type length"));
	hHeader->setLabel(nCol++, i18n("By value"));
    hHeader->setLabel(nCol++, i18n("Is defined"));
    hHeader->setLabel(nCol++, i18n("Array delimiter"));
    hHeader->setLabel(nCol++, i18n("Align"));
    hHeader->setLabel(nCol++, i18n("Storage"));
	hHeader->setLabel(nCol++, i18n("Element"));
	hHeader->setLabel(nCol++, i18n("Input function"));
	hHeader->setLabel(nCol++, i18n("Output function"));
	hHeader->setLabel(nCol++, i18n("Corresponding object"));
	hHeader->setLabel(nCol++, i18n("Owner"));

    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        m_pTableTypes->adjustColumn(nCol);
    }
	
	m_bPageTypesDisplaied = true;
}

void KPGSchema::displayViews(QWidget *widgetProperies)
{
	KPGSchemaPropPage *pPropertyPage = static_cast <KPGSchemaPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display views list
	QTable *pTableViews = pPropertyPage->m_pTableViews;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderViews->resultViews(), pTableViews);

    // field names
    QHeader* hHeader = pTableViews->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
    hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Owner"));
    hHeader->setLabel(nCol++, i18n("ACL"));

    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        pTableViews->adjustColumn(nCol);
    }
	
	m_bPageViewsDisplaied = true;
}

void KPGSchema::fillListOfObjectsForCodeCompletition(KPGOidNameList & listOfObjectOids)
{
    m_pFolderTables->fillListOfObjectOidsWithChildItems(listOfObjectOids);
	m_pFolderViews->fillListOfObjectOidsWithChildItems(listOfObjectOids);
	m_pFolderFunctions->fillListOfObjectOidsWithChildItems(listOfObjectOids);
}
