//
// C++ Implementation: kpgsequence
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgsequence.h"

// include files for Qt
#include <qtable.h>

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include "../kpgsqldialog.h"

#include "kpgconnection.h"
#include "kpgserver.h"
#include "kpgdatabase.h"
#include "kpgschema.h"
#include "kpgsequencesfolder.h"
#include "../PropertyPages/kpgsequenceproppage.h"
#include "../kpgsqldialog.h"


KPGSequence::KPGSequence(KPGSequencesFolder *parent, const QString name, pqxx::oid _oid)
 : KPGObject(parent, name, _oid)
{
	setPixmap(0, *m_pIconSequence);
	
	m_llLastValue = 0;
	m_llMinValue = 0; 
	m_llMaxValue = 0; 
	m_llCacheValue = 0; 
	m_bIsCycled = false; 
	m_iIncrementBy = 1;
}

KPGSequence::KPGSequence(KPGSequencesFolder *parent, KPGSequence *after, const QString name, pqxx::oid _oid)
 : KPGObject(parent, after, name, _oid)
{
	setPixmap(0, *m_pIconSequence);
	
	m_llLastValue = 0;
	m_llMinValue = 0; 
	m_llMaxValue = 0; 
	m_llCacheValue = 0; 
	m_bIsCycled = false; 
	m_iIncrementBy = 1;
}


KPGSequence::~KPGSequence()
{
}

void KPGSequence::setProperties(
	const QString strOwner, 
	const QString strACL,
	const QString strDescription
	)
{
	m_strOwner = strOwner;
	m_strACL = strACL;
	m_strDescription = strDescription;
}

void KPGSequence::setSeqValues(long long llLastValue, long long llMinValue, long long llMaxValue, long long llCacheValue, bool bIsCycled, int iIncrementBy)
{

	m_llLastValue = llLastValue;
	m_llMinValue = llMinValue; 
	m_llMaxValue = llMaxValue; 
	m_llCacheValue = llCacheValue; 
	m_bIsCycled = bIsCycled; 
	m_iIncrementBy = iIncrementBy;
}


void KPGSequence::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	KPGSequencePropPage *pPropertyPage = static_cast <KPGSequencePropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
	QHeader* hHeader = pTableProperties->horizontalHeader();
	
	hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  	pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Name of the sequence"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  	pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the sequence"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  	pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the sequence"));
	
	//--- ACL
	pPropertyPage->setACL(m_strACL);
	
	//--- Last value
	pTableProperties->setText(iRow, 0, i18n("Last value"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_llLastValue));
	pTableProperties->setText(iRow++, 2, i18n("Last counter value"));
	
	//--- Minimum value
	pTableProperties->setText(iRow, 0, i18n("Minimum value"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_llMinValue));
	pTableProperties->setText(iRow++, 2, i18n("Minimum counter value"));
	
	//--- Maximum value
	pTableProperties->setText(iRow, 0, i18n("Maximum value"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_llMaxValue));
	pTableProperties->setText(iRow++, 2, i18n("Maximum counter value"));
	
	//--- Cache value
	pTableProperties->setText(iRow, 0, i18n("Cache value"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_llCacheValue));
	pTableProperties->setText(iRow++, 2, i18n("Cache counter value"));
	
	//--- Is Cycled
	pTableProperties->setText(iRow, 0, i18n("Is Cycled"));
  	pTableProperties->setText(iRow, 1, (m_bIsCycled) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True, if counter value is cycled, when reaching max. value"));
	
	//--- Increment By
	pTableProperties->setText(iRow, 0, i18n("Increment By"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_iIncrementBy));
	pTableProperties->setText(iRow++, 2, i18n("Increment value for sequence"));
		
				
	for(int nCol = 0; nCol < 3; nCol++)
	{
		pTableProperties->adjustColumn(nCol);
	}
  
  displayIoStatistics(widgetProperies);
}

void KPGSequence::displayIoStatistics(QWidget *widgetProperies)
{
  QString strQuery("SELECT relname, blks_read, blks_hit FROM pg_catalog.pg_statio_all_sequences WHERE relid = ");
  strQuery.append(QString("%1;").arg(m_oid));
  
  KPGSequencePropPage *pPropertyPage = static_cast <KPGSequencePropPage *> (widgetProperies);
  
  //----------------------------------------------------------------------------
  //Display statistics information
  QTable *pTableStatistics = pPropertyPage->m_pTableIoStatistics;
  
  //--- Header field names
  QHeader* hHeader = pTableStatistics->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
  hHeader->setLabel(1, i18n("Value"));
  hHeader->setLabel(2, i18n("Description"));

  try
  {
    KPGConnection *pDbConnection = connection();
    pqxx::work pqxxXaction(*pDbConnection, "refresh sequence statistics");
    pqxx::result pqxxResultStatistics = pqxxXaction.Exec(strQuery.latin1());

    if(pqxxResultStatistics.size() != 1)
    {
      kdError() << k_funcinfo "Expect one row in result !" <<  endl;
      return;
    }
    
    if(text(0) != pqxxResultStatistics[0]["relname"].c_str())
    {
      kdDebug() << "KPGSequence::displayIoStatistics(): sequence name not match" << endl;
      return;
    }
    
    int iRow = 0;
    
    //--- blks_read
    pTableStatistics->setText(iRow, 0, i18n("blks_read"));
    pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["blks_read"].c_str());
    pTableStatistics->setText(iRow++, 2, i18n("Total number of disc blocks read"));
    
    //--- blks_hit
    pTableStatistics->setText(iRow, 0, i18n("blks_hit"));
    pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["blks_hit"].c_str());
    pTableStatistics->setText(iRow++, 2, i18n("Number of buffer hits"));
    
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
    dlg.exec();
    return;
  } 
  
  for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableStatistics->adjustColumn(nCol);
  }
}

// Refresh only view info, without childs objects
bool KPGSequence::refreshItem()
{
  // Get pointer to server for version info
  KPGSchema *pSchema = static_cast <KPGSchema *> (parent()->parent());
  KPGDatabase *pDatabase = static_cast <KPGDatabase *> (pSchema->parent());
  KPGServer *pServer = static_cast <KPGServer *> (pDatabase->parent());
  if(pServer->type() != nodeServer)
  {
    kdError() << "KPGSequence::refreshItem cannot find parent server." << endl;
    return "";
  }

  work pqxxXaction(* pDatabase->connection(), "refreshItem"); 
  
  // obtain sequence info
  QString strQuery("SELECT cl.oid, relname, description, pg_get_userbyid(relowner) AS seqowner, relacl ");
  strQuery.append("FROM pg_catalog.pg_class cl ");
  strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON des.objoid=cl.oid ");
  
  strQuery.append("WHERE cl.oid = " + QString("%1").arg(m_oid));
      
  try
  {
    pqxx::result pqxxResultSequences = pqxxXaction.Exec(strQuery.latin1());

    if(pqxxResultSequences.size() != 1)
    {
      kdError() << k_funcinfo "Expect one row in result !" <<  endl;
      return false;
    }
          
    setProperties(
      pqxxResultSequences[0]["seqowner"].c_str(), 
      pqxxResultSequences[0]["relacl"].c_str(),
      pqxxResultSequences[0]["description"].c_str()
      );
      
      
    if((pServer->versionMajor() >= 8) || ((pServer->versionMajor() == 7) && (pServer->versionMiddle() >= 4)))
    {           
     //-- Get seq. values
     strQuery = QString("SELECT last_value, min_value, max_value, cache_value, is_cycled, increment_by FROM  \"%1\".\"%2\";").arg(pSchema->text(0)).arg(pqxxResultSequences[0]["relname"].c_str());
     pqxx::result pqxxResultSeqValues = pqxxXaction.exec(strQuery.latin1());
        
     if (pqxxResultSeqValues.size() == 1)
     { 
        QString strLastValue(pqxxResultSeqValues[0]["last_value"].c_str());
        QString strMinValue(pqxxResultSeqValues[0]["min_value"].c_str()); 
        QString strMaxValue(pqxxResultSeqValues[0]["max_value"].c_str()); 
        QString strCacheValue(pqxxResultSeqValues[0]["cache_value"].c_str()); 
               
        long long llLastValue = strLastValue.toLongLong();
        long long llMinValue = strMinValue.toLongLong(); 
        long long llMaxValue = strMaxValue.toLongLong(); 
        long long llCacheValue = strCacheValue.toLongLong(); 
        bool bIsCycled; 
        int iIncrementBy;
    
        /*pqxxResultSeqValues[0]["last_value"].to(iLastValue);
        pqxxResultSeqValues[0]["min_value"].to(iMinValue);
        pqxxResultSeqValues[0]["max_value"].to(iMaxValue);
        pqxxResultSeqValues[0]["cache_value"].to(iCacheValue);*/
        pqxxResultSeqValues[0]["is_cycled"].to(bIsCycled);
        pqxxResultSeqValues[0]["increment_by"].to(iIncrementBy);
        
        setSeqValues(llLastValue, llMinValue, llMaxValue, llCacheValue, bIsCycled, iIncrementBy);
      }  
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
    dlg.exec();
    return false;
  } 
  
  return true;
}
