/***************************************************************************
                          kpgtable.cpp  -  description
                             -------------------
    begin                : � led 6 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpgtable.h"

// include files for Qt
#include <qtable.h>
 
// include files for KDE
#include <kiconloader.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgserver.h"
#include "kpgdatabase.h"
#include "kpgtablesfolder.h"
#include "kpgtablecolumnsfolder.h"
#include "kpgtableconstraintsfolder.h"
#include "kpgtableindexesfolder.h"
#include "kpgrulesfolder.h"
#include "kpgtriggersfolder.h"
#include "../PropertyPages/kpgtableproppage.h"
#include "../kpglinklabel.h"
#include "../kpgsqldialog.h"


KPGTable::KPGTable(KPGTablesFolder *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
  setPixmap(0, *m_pIconTable);
	
	m_pFolderColumns = new KPGTableColumnsFolder(this);
	m_pFolderConstraints = new KPGTableConstraintsFolder(this, m_pFolderColumns);
	m_pFolderIndexes = new KPGTableIndexesFolder(this, m_pFolderConstraints);
	m_pFolderRules = new KPGRulesFolder(this, m_pFolderIndexes);
	m_pFolderTriggers = new KPGTriggersFolder(this, m_pFolderRules);
}

KPGTable::KPGTable(KPGTablesFolder *parent, KPGTable *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconTable);
	
	m_pFolderColumns = new KPGTableColumnsFolder(this);
	m_pFolderConstraints = new KPGTableConstraintsFolder(this, m_pFolderColumns);
	m_pFolderIndexes = new KPGTableIndexesFolder(this, m_pFolderConstraints);
	m_pFolderRules = new KPGRulesFolder(this, m_pFolderIndexes);
	m_pFolderTriggers = new KPGTriggersFolder(this, m_pFolderRules);
}

KPGTable::~KPGTable()
{
}

bool KPGTable::refresh()
{
  KPGConnection *pDbConnection = connection();
	work pqxxXaction(*pDbConnection, "refresh");
	
	if(!m_pFolderColumns->refresh(pqxxXaction, m_oid))
		return false;
	
	if(!m_pFolderConstraints->refresh(pqxxXaction, m_oid))
		return false;
		
	if(!m_pFolderIndexes->refresh(pqxxXaction, m_oid))
		return false;
		
	if(!m_pFolderRules->refresh(pqxxXaction, m_oid))
		return false;
		
	if(!m_pFolderTriggers->refresh(pqxxXaction, m_oid))
		return false;
		
	return true;
}

bool KPGTable::refreshColumns(work &pqxxXaction)
{
    return m_pFolderColumns->refresh(pqxxXaction, m_oid);
}

bool KPGTable::refreshConstraints(work &pqxxXaction)
{
 	return m_pFolderConstraints->refresh(pqxxXaction, m_oid);
}

bool KPGTable::refreshIndexes(work &pqxxXaction)
{
    return m_pFolderIndexes->refresh(pqxxXaction, m_oid);
}   

bool KPGTable::refreshRules(work &pqxxXaction)
{
    return m_pFolderRules->refresh(pqxxXaction, m_oid);
}

bool KPGTable::refreshTriggers(work &pqxxXaction)
{
    return m_pFolderTriggers->refresh(pqxxXaction, m_oid);
}

// Refresh only table info, without childs objects
bool KPGTable::refreshItem()
{
	// Get pointer to server for version info
	KPGDatabase *pDatabase = static_cast <KPGDatabase *> (parent()->parent()->parent());
	KPGServer *pServer = static_cast <KPGServer *> (pDatabase->parent());
	
	work pqxxXaction(* pDatabase->connection(), "refreshItem"); 
	
	// obtain table info
	QString strQuery("SELECT rel.oid, relname, description, pg_get_userbyid(relowner) AS relowner, relacl, ");
	strQuery.append("relhasoids, relhassubclass, reltuples, conname, conkey ");
	
	if(pServer->versionMajor() >= 8)
		strQuery.append(", rel.reltablespace, ts.spcname ");
		
	strQuery.append("FROM pg_catalog.pg_class rel ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON des.objoid=rel.oid AND des.objsubid=0 ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_constraint c ON c.conrelid=rel.oid AND c.contype='p' ");
	
	if(pServer->versionMajor() >= 8)
		strQuery.append("LEFT JOIN pg_catalog.pg_tablespace ts ON rel.reltablespace=ts.oid ");
		
	strQuery.append("WHERE rel.oid = " + QString("%1").arg(m_oid));
		
	try
	{
		pqxx::result pqxxResultTables = pqxxXaction.Exec(strQuery.latin1());
	
		if(pqxxResultTables.size() != 1)
		{
            kdError() << k_funcinfo "Expect one row in result !" <<  endl;
            return false;
		}
		
		// Get refreshed properties
		bool bHasOids;
		bool bHasSubclass;
		double dEstimatedRows;
		
		pqxxResultTables[0]["relhasoids"].to(bHasOids);
		pqxxResultTables[0]["relhassubclass"].to(bHasSubclass);
		pqxxResultTables[0]["reltuples"].to(dEstimatedRows);
		
		if(pServer->versionMajor() <= 7)
		{
            setProperties(
                pqxxResultTables[0]["relowner"].c_str(), 
                pqxxResultTables[0]["relacl"].c_str(),
                pqxxResultTables[0]["description"].c_str(), 
                bHasOids,
                bHasSubclass,
                dEstimatedRows,
                pqxxResultTables[0]["conname"].c_str(),
                pqxxResultTables[0]["conkey"].c_str() 
                );
		}
		else
		{
            pqxx::oid oidTablespace;
            pqxxResultTables[0]["reltablespace"].to(oidTablespace);
            
            setProperties(
                pqxxResultTables[0]["relowner"].c_str(), 
                pqxxResultTables[0]["relacl"].c_str(),
                pqxxResultTables[0]["description"].c_str(), 
                bHasOids,
                bHasSubclass,
                dEstimatedRows,
                pqxxResultTables[0]["conname"].c_str(),
                pqxxResultTables[0]["conkey"].c_str(),
                oidTablespace,
                pqxxResultTables[0]["spcname"].c_str()
            );
		}
	}
	catch (const std::exception &e)
	{
		kdError() << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return false;
	} 
	
	return true;
}

// For PostgreSQL 7.x or older 
void KPGTable::setProperties(
	const QString strOwner, 
	const QString strACL,
	const QString strDescription,
	bool bHasOids,
	bool bHasSubclass,
	double dEstimatedRows,
	const QString strPrimaryKeyName,
	const QString strPrimaryKeyColNumbers 
	)
{
	m_strOwner = strOwner;
	m_strACL = strACL;
	m_strDescription = strDescription;
	
	m_bHasOids = bHasOids;
	m_bHasSubclass = bHasSubclass;
	m_dEstimatedRows = dEstimatedRows;
	m_strPrimaryKeyName = strPrimaryKeyName;
	m_strPrimaryKeyColNumbers = strPrimaryKeyColNumbers; 
	m_oidTablespace = 0;
}

// For PostgreSQL 8 or newer 
void KPGTable::setProperties(
	const QString strOwner, 
	const QString strACL,
	const QString strDescription,
	bool bHasOids,
	bool bHasSubclass,
	double dEstimatedRows,
	const QString strPrimaryKeyName,
	const QString strPrimaryKeyColNumbers,
	pqxx::oid oidTablespace,
  QString strTablespace  
	)
{
	m_strOwner = strOwner;
	m_strACL = strACL;
	m_strDescription = strDescription;
	m_bHasOids = bHasOids;
	m_bHasSubclass = bHasSubclass;
	m_dEstimatedRows = dEstimatedRows;
	m_strPrimaryKeyName = strPrimaryKeyName;
	m_strPrimaryKeyColNumbers = strPrimaryKeyColNumbers; 
	m_oidTablespace = oidTablespace;   
  m_strTablespace = strTablespace; 
}

void KPGTable::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	KPGTablePropPage *pPropertyPage = static_cast <KPGTablePropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header field names
  	QHeader* hHeader = pTableProperties->horizontalHeader();

  	hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
		
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  	pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Name of the table"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  	pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the table"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  	pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the table"));
	
	//--- ACL
	pPropertyPage->setACL(m_strACL);
	
	//--- Has OIDs
	pTableProperties->setText(iRow, 0, i18n("Has OIDs"));
  	pTableProperties->setText(iRow, 1, (m_bHasOids) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if we generate an OID for each row of the relation."));
	
	//--- Has subclass
	pTableProperties->setText(iRow, 0, i18n("Has subclass"));
  	pTableProperties->setText(iRow, 1, (m_bHasSubclass) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("At least one table inherits from this one"));
	
	//--- Estimated rows
	pTableProperties->setText(iRow, 0, i18n("Estimated rows"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_dEstimatedRows));
	pTableProperties->setText(iRow++, 2, i18n("Number of rows in the table. This is only an estimate used by the planner. It is updated by VACUUM, ANALYZE, and CREATE INDEX. "));
	
	//--- Primary key name
	pTableProperties->setText(iRow, 0, i18n("Primary key name"));
  	pTableProperties->setText(iRow, 1, m_strPrimaryKeyName);
	pTableProperties->setText(iRow++, 2, i18n("Primary key name"));
	
	//--- Primary key columns
	pTableProperties->setText(iRow, 0, i18n("Primary key columns"));
  	pTableProperties->setText(iRow, 1, m_strPrimaryKeyColNumbers);
	pTableProperties->setText(iRow++, 2, i18n("Primary key columns numbers"));
	
  //--- Tablespace
	pTableProperties->setText(iRow, 0, i18n("Tablespace"));
	KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Tablespace" );
	pLabel->setValues(m_oidTablespace, m_strTablespace);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Tablespace name"));
	
		
	for(int nCol = 0; nCol < 3; nCol++)
	{
		pTableProperties->adjustColumn(nCol);
	}
	
	displayColumns(widgetProperies);
	displayConstraints(widgetProperies);
	displayIndexes(widgetProperies);
	displayRules(widgetProperies);
	displayTriggers(widgetProperies);
  
	displayStatistics(widgetProperies);
	displayIoStatistics(widgetProperies);
}

void KPGTable::displayColumns(QWidget *widgetProperies)
{
	KPGTablePropPage *pPropertyPage = static_cast <KPGTablePropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display columns list
	QTable *pTableColumns = pPropertyPage->m_pTableColumns;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderColumns->resultColumns(), pTableColumns);

    // field names
    QHeader* hHeader = pTableColumns->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Type"));
	hHeader->setLabel(nCol++, i18n("Type length"));
	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Statistics target"));
    hHeader->setLabel(nCol++, i18n("Attribute number"));
    hHeader->setLabel(nCol++, i18n("Number of dimensions"));
    hHeader->setLabel(nCol++, i18n("Type mode"));
    hHeader->setLabel(nCol++, i18n("By value"));
    hHeader->setLabel(nCol++, i18n("Not null"));
    hHeader->setLabel(nCol++, i18n("Has default"));
    hHeader->setLabel(nCol++, i18n("Is dropped"));
    hHeader->setLabel(nCol++, i18n("Is local"));
    hHeader->setLabel(nCol++, i18n("Storage"));
    hHeader->setLabel(nCol++, i18n("Inherited count"));
    hHeader->setLabel(nCol++, i18n("Default value"));
    hHeader->setLabel(nCol++, i18n("Namespace"));
    
    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        pTableColumns->adjustColumn(nCol);
    }
}

void KPGTable::displayConstraints(QWidget *widgetProperies)
{
	KPGTablePropPage *pPropertyPage = static_cast <KPGTablePropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display constraints list
	QTable *pTableConstraints = pPropertyPage->m_pTableConstraints;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderConstraints->resultConstraints(), pTableConstraints);

    // field names
    QHeader* hHeader = pTableConstraints->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Type"));
    hHeader->setLabel(nCol++, i18n("Description"));
    hHeader->setLabel(nCol++, i18n("Is deferrable"));
    hHeader->setLabel(nCol++, i18n("Is deferred"));
    hHeader->setLabel(nCol++, i18n("Namespace"));
    
    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        pTableConstraints->adjustColumn(nCol);
    }
}

void KPGTable::displayIndexes(QWidget *widgetProperies)
{
	KPGTablePropPage *pPropertyPage = static_cast <KPGTablePropPage *> (widgetProperies);
	
	KPGDatabase *pDatabase = static_cast <KPGDatabase *> (parent()->parent()->parent());
	KPGServer *pServer = static_cast <KPGServer *> (pDatabase->parent());
	
	//----------------------------------------------------------------------------
	//Display indexes list
	QTable *pTableIndexes = pPropertyPage->m_pTableIndexes;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderIndexes->resultIndexes(), pTableIndexes, true, (pServer->versionMajor() >= 8) ? 8 : 7); // TODO test it !

    // field names
    QHeader* hHeader = pTableIndexes->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Type"));
    hHeader->setLabel(nCol++, i18n("Description"));
    hHeader->setLabel(nCol++, i18n("Index key"));
    hHeader->setLabel(nCol++, i18n("Is Unique"));
    hHeader->setLabel(nCol++, i18n("Is Primary"));
    hHeader->setLabel(nCol++, i18n("Is Clustered"));
    if(pServer->versionMajor() >= 8) hHeader->setLabel(nCol++, i18n("Tablespace"));
            
    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        pTableIndexes->adjustColumn(nCol);
    }
}

void KPGTable::displayRules(QWidget *widgetProperies)
{
	KPGTablePropPage *pPropertyPage = static_cast <KPGTablePropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display rules list
	QTable *pTableRules = pPropertyPage->m_pTableRules;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderRules->resultRules(), pTableRules);

	// field names
    QHeader* hHeader = pTableRules->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Is Instead"));
	hHeader->setLabel(nCol++, i18n("Event type"));
    hHeader->setLabel(nCol++, i18n("Qualifying Condition"));
    hHeader->setLabel(nCol++, i18n("Namespace"));
    
    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        pTableRules->adjustColumn(nCol);
    }
}

void KPGTable::displayTriggers(QWidget *widgetProperies)
{
	KPGTablePropPage *pPropertyPage = static_cast <KPGTablePropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display triggers list
	QTable *pTableTriggers = pPropertyPage->m_pTableTriggers;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderTriggers->resultTriggers(), pTableTriggers);
	
	// field names
    QHeader* hHeader = pTableTriggers->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Is Enabled"));
    hHeader->setLabel(nCol++, i18n("Is Deferrable"));
    hHeader->setLabel(nCol++, i18n("Is initially Deferred"));
    hHeader->setLabel(nCol++, i18n("Number of arguments"));
    hHeader->setLabel(nCol++, i18n("Arguments"));
        hHeader->setLabel(nCol++, i18n("Namespace"));
    
    for(int nCol = 0; nCol < nTableCols; nCol++)
    {
        pTableTriggers->adjustColumn(nCol);
    }
}

void KPGTable::displayStatistics(QWidget *widgetProperies)
{
    QString strQuery("SELECT relname, seq_scan, seq_tup_read, idx_scan, idx_tup_fetch, n_tup_ins, n_tup_upd, n_tup_del FROM pg_catalog.pg_stat_all_tables WHERE relid = ");
    strQuery.append(QString("%1;").arg(m_oid));
    
    KPGTablePropPage *pPropertyPage = static_cast <KPGTablePropPage *> (widgetProperies);
    
    //----------------------------------------------------------------------------
    //Display statistics information
    QTable *pTableStatistics = pPropertyPage->m_pTableStatistics;
    
    //--- Header field names
    QHeader* hHeader = pTableStatistics->horizontalHeader();
    
    hHeader->setLabel(0, i18n("Name"));
    hHeader->setLabel(1, i18n("Value"));
    hHeader->setLabel(2, i18n("Description"));
    
    try
    {
        KPGConnection *pDbConnection = connection();
        pqxx::work pqxxXaction(*pDbConnection, "refresh table statistics");
        pqxx::result pqxxResultStatistics = pqxxXaction.Exec(strQuery.latin1());
    
        if(pqxxResultStatistics.size() != 1)
        {
        	kdError() << k_funcinfo "Expect one row in result !" <<  endl;
        	return;
        }
        
        if(text(0) != pqxxResultStatistics[0]["relname"].c_str())
        {
        	kdDebug() << "KPGTable::displayStatistics(): table name not match" << endl;
        	return;
        }
        
        int iRow = 0;
        
        //--- seq_scan
        pTableStatistics->setText(iRow, 0, i18n("seq_scan"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["seq_scan"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Total number of sequential scans"));
        
        //--- seq_scan
        pTableStatistics->setText(iRow, 0, i18n("seq_tup_read"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["seq_tup_read"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Total number of tuples read"));
        
        //--- idx_scan
        pTableStatistics->setText(iRow, 0, i18n("idx_scan"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_scan"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Total number of index scans"));
        
        //--- idx_scan
        pTableStatistics->setText(iRow, 0, i18n("idx_tup_fetch"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_tup_fetch"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Total number of index tuples read"));
        
        //--- n_tup_ins
        pTableStatistics->setText(iRow, 0, i18n("n_tup_ins"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["n_tup_ins"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Total number of tuples inserted"));
        
        //--- n_tup_upd
        pTableStatistics->setText(iRow, 0, i18n("n_tup_upd"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["n_tup_upd"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Total number of tuples updated"));
        
        //--- n_tup_del
        pTableStatistics->setText(iRow, 0, i18n("n_tup_del"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["n_tup_del"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Total number of tuples deleted"));
        
    
    }
    catch (const std::exception &e)
    {
        kdError() << e.what() << endl;
        KPGSqlDialog dlg(0, strQuery, e.what());
        dlg.exec();
        return;
    } 
    
    for(int nCol = 0; nCol < 3; nCol++)
    {
        pTableStatistics->adjustColumn(nCol);
    }
}

void KPGTable::displayIoStatistics(QWidget *widgetProperies)
{
    QString strQuery("SELECT relname, heap_blks_read, heap_blks_hit, idx_blks_read, idx_blks_hit, toast_blks_read, toast_blks_hit, tidx_blks_read, tidx_blks_hit FROM \
pg_catalog.pg_statio_all_tables WHERE relid = ");
    strQuery.append(QString("%1;").arg(m_oid));
    
    KPGTablePropPage *pPropertyPage = static_cast <KPGTablePropPage *> (widgetProperies);
    
    //----------------------------------------------------------------------------
    //Display statistics information
    QTable *pTableStatistics = pPropertyPage->m_pTableIoStatistics;
    
    //--- Header field names
    QHeader* hHeader = pTableStatistics->horizontalHeader();
    
    hHeader->setLabel(0, i18n("Name"));
    hHeader->setLabel(1, i18n("Value"));
    hHeader->setLabel(2, i18n("Description"));
    
    try
    {
        KPGConnection *pDbConnection = connection();
        pqxx::work pqxxXaction(*pDbConnection, "refresh table statistics");
        pqxx::result pqxxResultStatistics = pqxxXaction.Exec(strQuery.latin1());
    
        if(pqxxResultStatistics.size() != 1) // querying pq_xactloct return no rows... Why ?
        {
            kdError() << k_funcinfo "Expect one row in result !" <<  endl;
            return;
        }
        
        if(text(0) != pqxxResultStatistics[0]["relname"].c_str())
        {
            kdDebug() << "KPGTable::displayStatistics(): table name not match" << endl;
            return;
        }
        
        int iRow = 0;
        
        //--- heap_blks_read
        pTableStatistics->setText(iRow, 0, i18n("heap_blks_read"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["heap_blks_read"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Total number of disc blocks read"));
        
        //--- heap_blks_hit
        pTableStatistics->setText(iRow, 0, i18n("heap_blks_hit"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["heap_blks_hit"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Number of buffer hits"));
        
        //--- idx_blks_read
        pTableStatistics->setText(iRow, 0, i18n("idx_blks_read"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_blks_read"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Total number of disc block read"));
        
        //--- idx_blks_hit
        pTableStatistics->setText(iRow, 0, i18n("idx_blks_hit"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_blks_hit"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Number of buffer hits in all indexes"));
        
        //--- toast_blks_read
        pTableStatistics->setText(iRow, 0, i18n("toast_blks_read"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["toast_blks_read"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Number of disc block read from table auxiliary TOAST table"));
        
        //--- n_tup_upd
        pTableStatistics->setText(iRow, 0, i18n("toast_blks_hit"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["toast_blks_hit"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Number of bufer hits from table auxiliary TOAST table"));
        
        //--- tidx_blks_read
        pTableStatistics->setText(iRow, 0, i18n("tidx_blks_read"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["tidx_blks_read"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Number of disc block read from table auxiliary TOAST table's index"));
        
        //--- tidx_blks_read
        pTableStatistics->setText(iRow, 0, i18n("tidx_blks_hit"));
        pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["tidx_blks_hit"].c_str());
        pTableStatistics->setText(iRow++, 2, i18n("Number of bufer hits from table auxiliary TOAST table's index"));
    }
    catch (const std::exception &e)
    {
        kdError() << e.what() << endl;
        KPGSqlDialog dlg(0, strQuery, e.what());
        dlg.exec();
        return;
    } 
    
    for(int nCol = 0; nCol < 3; nCol++)
    {
        pTableStatistics->adjustColumn(nCol);
    }
}

