/***************************************************************************
                          kpgtable.h  -  description
                             -------------------
    begin                : � led 6 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPGTABLE_H
#define KPGTABLE_H

#include "kpgobject.h"

#include "kpgconnection.h"

class KPGTablesFolder;
class KPGTableColumnsFolder;
class KPGTableConstraintsFolder;
class KPGTableIndexesFolder;
class KPGRulesFolder;
class KPGTriggersFolder;

/**Item, that represent database table
  *@author Lumir Vanek
  */

class KPGTable : public KPGObject
{
public: 
	KPGTable(KPGTablesFolder *, const QString, pqxx::oid);
	KPGTable(KPGTablesFolder *, KPGTable *, const QString, pqxx::oid);
	~KPGTable();

  	virtual eNodeType type() const { return nodeTable; }
  	
  	// Get SQL object type 
  	virtual const char * getObjectType() const { return "TABLE"; }
  
	bool refreshColumns(work &);
	bool refreshConstraints(work &);
	bool refreshIndexes(work &);
	bool refreshRules(work &);
	bool refreshTriggers(work &);
	virtual bool refresh();
  	bool refreshItem();
	
	void setProperties(const QString, const QString, const QString, bool, bool, double, const QString, const QString);
	void setProperties(const QString, const QString, const QString, bool, bool, double, const QString, const QString, pqxx::oid, const QString);
	void displayProperties(QWidget *);
  
	KPGTableColumnsFolder * getTableColumnsFolder() { return m_pFolderColumns; }
	KPGTableConstraintsFolder * getTableConstraintsFolder() { return m_pFolderConstraints; }
	KPGTableIndexesFolder * getTableIndexesFolder() { return m_pFolderIndexes; }
	KPGRulesFolder * getRulesFolder() { return m_pFolderRules; }
	KPGTriggersFolder * getTriggersFolder() { return m_pFolderTriggers; }
	
	const QString & primaryKeyName() const { return m_strPrimaryKeyName; }
	const QString & owner() const { return m_strOwner; }
	const QString & tablespace() const { return m_strTablespace; }
	
	void displayStatistics(QWidget *);
	void displayIoStatistics(QWidget *);
  
protected:
	void displayColumns(QWidget *);	
	void displayConstraints(QWidget *);
	void displayIndexes(QWidget *);	
	void displayRules(QWidget *);	
	void displayTriggers(QWidget *);	
	  
protected:
	// Table properties
	QString m_strOwner;
	QString m_strACL;
	bool m_bHasOids;
	bool m_bHasSubclass;
	double m_dEstimatedRows;
	QString m_strPrimaryKeyName;
	QString m_strPrimaryKeyColNumbers;
	pqxx::oid m_oidTablespace;   // For PostgreSQL 8 or newer 
	QString m_strTablespace;     // For PostgreSQL 8 or newer
	
	// Folders with owned objects
  	KPGTableColumnsFolder *m_pFolderColumns;
	KPGTableConstraintsFolder *m_pFolderConstraints;
	KPGTableIndexesFolder *m_pFolderIndexes;
	KPGRulesFolder *m_pFolderRules;
	KPGTriggersFolder *m_pFolderTriggers;
};

#endif
