/***************************************************************************
                          kpgtablecolumnsfolder  -  description
                             -------------------
    begin                : � no 5 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpgtablecolumnsfolder.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "kpgconnection.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "kpgtablecolumn.h"
#include "../kpgsqldialog.h"

KPGTableColumnsFolder::KPGTableColumnsFolder(KPGTable *parent)
 : KPGTableItemsFolder(parent, i18n("Columns"))
{
}

KPGTableColumnsFolder::KPGTableColumnsFolder(KPGTable *parent, KPGTableItemsFolder *after)
 : KPGTableItemsFolder(parent, after, i18n("Columns"))
{
}

KPGTableColumnsFolder::~KPGTableColumnsFolder()
{
}


bool KPGTableColumnsFolder::refresh(work &pqxxXaction, pqxx::oid oidTable)
{
	// delete all child items (databases)
	while (QListViewItem * pItem = firstChild())
		delete pItem;
	
	QString strQuery("SELECT a.attname, atttypid, ty.typname, ty.typbasetype, a.attlen, ");
		
	strQuery.append("description, a.attstattarget, a.attnum, a.attndims, a.atttypmod, a.attbyval, a.attnotnull, a.atthasdef, a.attisdropped, a.attislocal, CASE WHEN a.attstorage = 'p' THEN 'PLAIN' WHEN a.attstorage = 'e' THEN 'EXTERNAL' WHEN a.attstorage = 'm' THEN 'MAIN' WHEN a.attstorage = 'x' THEN 'EXTENDED' END AS attstorage, a.attinhcount, ");
		
	strQuery.append("def.adsrc, tn.nspname");
			
	strQuery.append(" FROM pg_catalog.pg_attribute a");
	strQuery.append(" JOIN pg_catalog.pg_type ty ON ty.oid=a.atttypid");
	strQuery.append(" JOIN pg_catalog.pg_namespace tn ON tn.oid=ty.typnamespace");
	strQuery.append(" JOIN pg_catalog.pg_class cl ON cl.oid=a.attrelid");
	//strQuery.append(" JOIN pg_catalog.pg_namespace na ON na.oid=cl.relnamespace");
	strQuery.append(" LEFT OUTER JOIN pg_catalog.pg_type et ON et.oid=ty.typelem");
	strQuery.append(" LEFT OUTER JOIN pg_catalog.pg_attrdef def ON adrelid=a.attrelid AND adnum=a.attnum");
	strQuery.append(" LEFT OUTER JOIN pg_catalog.pg_description des ON des.objoid=attrelid AND des.objsubid=attnum");
	strQuery.append(" WHERE attrelid = ");
	strQuery.append(QString("%1").arg(oidTable));
	strQuery.append(" AND attisdropped IS FALSE");
	strQuery.append(" ORDER BY attnum");
			
	try
	{
		m_pqxxResultColumns = pqxxXaction.exec(strQuery.latin1());
			
		KPGTableColumn *pgTableColumn = 0;
			
		for (result::size_type i = 0; i != m_pqxxResultColumns.size(); ++i)
		{
			if(pgTableColumn == 0)
				pgTableColumn = new KPGTableColumn(this, m_pqxxResultColumns[i]["attname"].c_str());
			else
				pgTableColumn = new KPGTableColumn(this, pgTableColumn, m_pqxxResultColumns[i]["attname"].c_str());
			
			pqxx::oid oidType = 0;
			pqxx::oid oidBaseType = 0;
			bool bByVal;
			bool bNotNull;
			bool bHasDef;
			bool bIsDropped;
			bool bIsLocal;
			long lStatTarget;
			int iAttLen;
			int iAttNum; 
			int iAttNdims;
			int iAttTypmod;
			int iInhCount;
				
			m_pqxxResultColumns[i]["atttypid"].to(oidType);
			m_pqxxResultColumns[i]["typbasetype"].to(oidBaseType);
			m_pqxxResultColumns[i]["attbyval"].to(bByVal);
			m_pqxxResultColumns[i]["attnotnull"].to(bNotNull);
			m_pqxxResultColumns[i]["atthasdef"].to(bHasDef);
			m_pqxxResultColumns[i]["attisdropped"].to(bIsDropped);
			m_pqxxResultColumns[i]["attislocal"].to(bIsLocal);
			m_pqxxResultColumns[i]["attstattarget"].to(lStatTarget);
			m_pqxxResultColumns[i]["attlen"].to(iAttLen);
			m_pqxxResultColumns[i]["attnum"].to(iAttNum);
			m_pqxxResultColumns[i]["attndims"].to(iAttNdims);
			m_pqxxResultColumns[i]["atttypmod"].to(iAttTypmod);
			m_pqxxResultColumns[i]["attinhcount"].to(iInhCount);
				
			pgTableColumn->setProperties(
					oidType,
					oidBaseType,
					m_pqxxResultColumns[i]["typname"].c_str(),
					m_pqxxResultColumns[i]["description"].c_str(),
					lStatTarget,
					iAttLen,
					iAttNum, 
					iAttNdims,
					iAttTypmod,
					bByVal, 
					bNotNull, 
					bHasDef, 
					bIsDropped, 
					bIsLocal, 
					m_pqxxResultColumns[i]["attstorage"].c_str(),
					iInhCount, 
					m_pqxxResultColumns[i]["adsrc"].c_str(),
					m_pqxxResultColumns[i]["nspname"].c_str()
					);
			}
	}
	catch (const std::exception &e)
	{
		kdError() << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return false;
	} 
	
	return true;
}

void KPGTableColumnsFolder::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  	pPropertyPage->setLabelText(i18n("Columns &list:"));
	
	//----------------------------------------------------------------------------
	//Display columns list
	QTable *pTableColumns = pPropertyPage->m_pTableItems;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultColumns, pTableColumns);

	// field names
  	QHeader* hHeader = pTableColumns->horizontalHeader();

  	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Type"));
	hHeader->setLabel(nCol++, i18n("Type length"));
	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Statistics target"));
	hHeader->setLabel(nCol++, i18n("Attribute number"));
	hHeader->setLabel(nCol++, i18n("Dimensions"));
	hHeader->setLabel(nCol++, i18n("Type mode"));
	hHeader->setLabel(nCol++, i18n("By value"));
	hHeader->setLabel(nCol++, i18n("Not null"));
	hHeader->setLabel(nCol++, i18n("Has default"));
	hHeader->setLabel(nCol++, i18n("Is dropped"));
	hHeader->setLabel(nCol++, i18n("Is local"));
	hHeader->setLabel(nCol++, i18n("Storage"));
	hHeader->setLabel(nCol++, i18n("Inherited count"));
	hHeader->setLabel(nCol++, i18n("Default value"));
	hHeader->setLabel(nCol++, i18n("Namespace"));

	for(int nCol = 0; nCol < nTableCols; nCol++)
	{
		pTableColumns->adjustColumn(nCol);
	}
}

void KPGTableColumnsFolder::fillListBoxWithChildItems(QListBox *pListBox, bool bIncludeSystemColumns)
{
	QListViewItem * pItem = firstChild();
	while(pItem)
	{
		KPGTableColumn *pTableColumn = static_cast <KPGTableColumn *> (pItem);
		
		if(bIncludeSystemColumns || (pTableColumn->attNum() >= 0))
		pListBox->insertItem(* pItem->pixmap(0), pItem->text(0));   
		
		pItem = pItem->nextSibling();
	}
}

void KPGTableColumnsFolder::fillListBoxWithChildItems(KListBox *pListBox, bool bIncludeSystemColumns)
{
	QListViewItem * pItem = firstChild();
	while(pItem)
	{
		KPGTableColumn *pTableColumn = static_cast <KPGTableColumn *> (pItem);
		
		if(bIncludeSystemColumns || (pTableColumn->attNum() >= 0))
		pListBox->insertItem(* pItem->pixmap(0), pItem->text(0));   
		
		pItem = pItem->nextSibling();
	}
}

void KPGTableColumnsFolder::fillComboBoxWithChildItems(KComboBox *pComboBox, bool bIncludeSystemColumns)
{
	QListViewItem * pItem = firstChild();
	while(pItem)
	{
		KPGTableColumn *pTableColumn = static_cast <KPGTableColumn *> (pItem);
		
		if(bIncludeSystemColumns || (pTableColumn->attNum() >= 0))
		pComboBox->insertItem(* pItem->pixmap(0), pItem->text(0));   
		
		pItem = pItem->nextSibling();
	}
}

void KPGTableColumnsFolder::fillListColumnWizInfoWithChildItems(KPGListTableColumnWizInfo & listColumnWizInfo, bool bIncludeSystemColumns)
{
	QListViewItem * pItem = firstChild();
	while(pItem)
	{
		KPGTableColumn *pTableColumn = static_cast <KPGTableColumn *> (pItem);
		
		if(bIncludeSystemColumns || (pTableColumn->attNum() > 0))
			{
				listColumnWizInfo.append(new KPGTableColumnWizInfo(
						pTableColumn->text(0), 
						pTableColumn->typName(),
						pTableColumn->iAttNdims(),
						pTableColumn->attNum(),
						*pTableColumn->pixmap(0)
						));
			}
		
		pItem = pItem->nextSibling();
	}
}
