//
// C++ Implementation: kpgtableconstraint
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtableconstraint.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgtableconstraintsfolder.h"
#include "kpgdatabase.h"
#include "../kpgsqldialog.h"

KPGTableConstraint::KPGTableConstraint(KPGTableConstraintsFolder *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
}

KPGTableConstraint::KPGTableConstraint(KPGTableConstraintsFolder *parent, KPGTableConstraint *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
}

KPGTableConstraint::~KPGTableConstraint()
{
}

// Refresh only constraint info, without childs objects
bool KPGTableConstraint::refreshItem()
{
	// Get pointer to owner table and database
	KPGTable *pTable = static_cast <KPGTable *> (parent()->parent());
	KPGDatabase *pDatabase = static_cast <KPGDatabase *> (pTable->parent()->parent()->parent());
		
	work pqxxXaction(* pDatabase->connection(), "refreshItem"); 
	
	QString strQuery("SELECT c.oid, conname, CASE contype WHEN 'p' THEN 'Primary Key' WHEN 'f' THEN 'Foreign Key' WHEN 'u' THEN 'Unique' WHEN 'c' THEN 'Check' END AS contypedesc, description, condeferrable, condeferred, nspname "); 
		
	strQuery.append("FROM pg_catalog.pg_constraint c ");
	
	strQuery.append("JOIN pg_catalog.pg_class cl ON cl.oid=conrelid ");
	strQuery.append("JOIN pg_catalog.pg_namespace nl ON nl.oid=relnamespace ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON des.objoid=c.oid ");
		
	strQuery.append("WHERE conrelid = ");
	strQuery.append(QString("%1").arg(pTable->oid()));
	strQuery.append(" AND c.oid=");
	strQuery.append(QString("%1").arg(m_oid));
	
	try
	{
    	pqxx::result pqxxResultConstraints = pqxxXaction.Exec(strQuery.latin1());

		if(pqxxResultConstraints.size() != 1)
		{
			kdError() << k_funcinfo "Expect one row in result !" <<  endl;
			return false;
		}
		
		bool bIsDeferrable;
		bool bIsDeferred;
				
		pqxxResultConstraints[0]["condeferrable"].to(bIsDeferrable);
		pqxxResultConstraints[0]["condeferred"].to(bIsDeferred);
							
		setProperties(
				pqxxResultConstraints[0]["description"].c_str(), 
				bIsDeferrable,
				bIsDeferred,
				pqxxResultConstraints[0]["nspname"].c_str()
				);
		
	}
	catch (const std::exception &e)
	{
		kdError() << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return false;
	} 
	
	return true;
}


void KPGTableConstraint::setProperties(
		const QString strDescription,
		bool bIsDeferrable,
		bool bIsDeferred,
		const QString strNspName	
	)
{
	m_strDescription = strDescription;
	m_bIsDeferrable = bIsDeferrable;
	m_bIsDeferred = bIsDeferred;
	m_strNspName = strNspName;	
}
