//
// C++ Implementation: kpgtableindex
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtableindex.h"

// include files for Qt
#include <qtable.h>

// include files for KDE
#include <kdebug.h>
#include <klocale.h>

#include "kpgtableindexesfolder.h"
#include "../kpgsqldialog.h"
#include "kpgdatabase.h"
#include "kpgserver.h"
#include "../kpgsqldialog.h"

KPGTableIndex::KPGTableIndex(KPGTableIndexesFolder *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
		setPixmap(0, *m_pIconFolderRed);
}

KPGTableIndex::KPGTableIndex(KPGTableIndexesFolder *parent, KPGTableIndex *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
		setPixmap(0, *m_pIconFolderRed);
}


KPGTableIndex::~KPGTableIndex()
{
}

// Refresh only index info, without childs objects
bool KPGTableIndex::refreshItem()
{
	// Get pointer to owner table and database
	KPGTable *pTable = static_cast <KPGTable *> (parent()->parent());
	KPGDatabase *pDatabase = static_cast <KPGDatabase *> (pTable->parent()->parent()->parent());
	KPGServer *pServer = static_cast <KPGServer *> (pDatabase->parent());
		
	work pqxxXaction(* pDatabase->connection(), "refreshItem"); 
	
	QString strQuery("SELECT cls.oid, cls.relname AS idxname, CASE contype WHEN 'p' THEN 'Primary Key' ELSE 'Index' END AS idxtypedesc, CASE contype WHEN 'p' THEN despkey.description ELSE desidx.description END AS description, indkey, indisunique, indisprimary, indisclustered, pg_get_indexdef(cls.oid) AS indexdef ");
	
	if(pServer->versionMajor() >= 8)
		strQuery.append(", cls.reltablespace, ts.spcname ");
		
	strQuery.append("FROM pg_catalog.pg_index idx JOIN pg_class cls ON cls.oid=indexrelid JOIN pg_am am ON am.oid=cls.relam ");
	
	strQuery.append("LEFT JOIN pg_catalog.pg_depend dep ON (dep.classid = cls.tableoid AND dep.objid = cls.oid AND dep.refobjsubid = '0') ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_constraint con ON (con.tableoid = dep.refclassid AND con.oid = dep.refobjid)   ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description desidx ON desidx.objoid=cls.oid ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description despkey ON (despkey.objoid=con.oid AND despkey.objsubid = 0) ");
	
	if(pServer->versionMajor() >= 8)
		strQuery.append("LEFT JOIN pg_catalog.pg_tablespace ts ON cls.reltablespace=ts.oid ");
	
	strQuery.append("WHERE indrelid =  ");
	strQuery.append(QString("%1").arg(pTable->oid()));
	strQuery.append(" AND cls.oid=");
	strQuery.append(QString("%1").arg(m_oid));
	
	try
	{
    	pqxx::result pqxxResultIndexes = pqxxXaction.Exec(strQuery.latin1());

		if(pqxxResultIndexes.size() != 1)
		{
			kdError() << k_funcinfo "Expect one row in result !" <<  endl;
			return false;
		}
		
		bool bIsUnique;
		bool bIsPrimary;
		bool bIsClustered;
									
		pqxxResultIndexes[0]["indisunique"].to(bIsUnique);
		pqxxResultIndexes[0]["indisprimary"].to(bIsPrimary);
		pqxxResultIndexes[0]["indisclustered"].to(bIsClustered);
				
		if(pServer->versionMajor() <= 7)
		{
			setProperties(
					pqxxResultIndexes[0]["description"].c_str(), 
					pqxxResultIndexes[0]["indkey"].c_str(),
					bIsUnique,
					bIsPrimary,
					bIsClustered,
					pqxxResultIndexes[0]["indexdef"].c_str()
					);
			}
			else
			{
				pqxx::oid oidTablespace;
				pqxxResultIndexes[0]["reltablespace"].to(oidTablespace);
					
				setProperties(
					pqxxResultIndexes[0]["description"].c_str(), 
					pqxxResultIndexes[0]["indkey"].c_str(),
					bIsUnique,
					bIsPrimary,
					bIsClustered,
					oidTablespace,
					pqxxResultIndexes[0]["spcname"].c_str(),
					pqxxResultIndexes[0]["indexdef"].c_str()
					);
			}
		
	}
	catch (const std::exception &e)
	{
		kdError() << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return false;
	} 
	
	refreshMapIndexKey(pTable->oid(), pqxxXaction);
	
	return true;
}

// Refresh map of index columns
bool KPGTableIndex::refreshMapIndexKey(pqxx::oid oidTable, pqxx::work & pqxxXaction)
{
	m_mapIndexKey.clear();
	if(m_strIndKey.length() == 0) return true;
	QString strIn(m_strIndKey.replace(" ", ", "));

	QString strQuery("SELECT a.attnum, a.attname ");
	strQuery.append("FROM pg_catalog.pg_attribute a ");
	strQuery.append("WHERE attrelid = ");
	strQuery.append(QString("%1").arg(oidTable)); 
	strQuery.append(" AND a.attnum IN(");
	strQuery.append(strIn);
	strQuery.append(") ORDER BY a.attnum;");
	 
	try
	{
		pqxx::result  pqxxResultColumns = pqxxXaction.exec(strQuery.latin1());
			
		for (result::size_type i = 0; i != pqxxResultColumns.size(); ++i)
		{
			int iAttNum; 
			pqxxResultColumns[i]["attnum"].to(iAttNum);
			QString strColumnName(pqxxResultColumns[i]["attname"].c_str());
			
			m_mapIndexKey[iAttNum] = strColumnName;
		}
	}
	catch (const std::exception &e)
	{
		kdError() << k_funcinfo << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return false;
	} 
	
	return true;
}

// For PostgreSQL 7.x or older
void KPGTableIndex::setProperties(
		const QString strDescription,
		const QString strIndKey,
		bool bIsUnique,
		bool bIsPrimary,
		bool bIsClustered,
    const QString &strIndexdef
	)
{
	m_strDescription = strDescription;
	m_strIndKey = strIndKey;
	m_bIsUnique = bIsUnique;
	m_bIsPrimary = bIsPrimary;
	m_bIsClustered = bIsClustered;
  m_oidTablespace = 0;
  m_strIndexdef = strIndexdef;
}

// For PostgreSQL 8 or newer
void KPGTableIndex::setProperties(
    const QString strDescription,
    const QString strIndKey,
    bool bIsUnique,
    bool bIsPrimary,
    bool bIsClustered,
    pqxx::oid oidTablespace,    
    QString strTablespace,
    const QString &strIndexdef     
  )
{
  m_strDescription = strDescription;
  m_strIndKey = strIndKey;
  m_bIsUnique = bIsUnique;
  m_bIsPrimary = bIsPrimary;
  m_bIsClustered = bIsClustered;
  m_oidTablespace = oidTablespace;
  m_strTablespace = strTablespace;
  m_strIndexdef = strIndexdef;
}

void KPGTableIndex::displayStatistics(QTable *pTableStatistics)
{
	QString strQuery("SELECT indexrelname, idx_scan, idx_tup_read, idx_tup_fetch FROM pg_catalog.pg_stat_all_indexes WHERE indexrelid = ");
	strQuery.append(QString("%1;").arg(m_oid));
	
	//--- Header field names
	QHeader* hHeader = pTableStatistics->horizontalHeader();
	
	hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	try
	{
		KPGConnection *pDbConnection = connection();
		pqxx::work pqxxXaction(*pDbConnection, "refresh table statistics");
		pqxx::result pqxxResultStatistics = pqxxXaction.Exec(strQuery.latin1());
	
		if(pqxxResultStatistics.size() != 1)
		{
			kdError() << k_funcinfo "Expect one row in result !" <<  endl;
			return;
		}
		
		if(text(0) != pqxxResultStatistics[0]["indexrelname"].c_str())
		{
		kdDebug() << "KPGTable::displayStatistics(): table name not match" << endl;
		return;
		}
		
		int iRow = 0;
		
		//--- seq_scan
		pTableStatistics->setText(iRow, 0, i18n("idx_scan"));
		pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_scan"].c_str());
		pTableStatistics->setText(iRow++, 2, i18n("Total number of index scans"));
		
		//--- seq_scan
		pTableStatistics->setText(iRow, 0, i18n("idx_tup_read"));
		pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_tup_read"].c_str());
		pTableStatistics->setText(iRow++, 2, i18n("Total number of tuples read"));
		
		//--- idx_tup_fetch
		pTableStatistics->setText(iRow, 0, i18n("idx_tup_fetch"));
		pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_tup_fetch"].c_str());
		pTableStatistics->setText(iRow++, 2, i18n("Total number of sucessfully fetched heap tuples"));
	}
	catch (const std::exception &e)
	{
		kdError() << k_funcinfo << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return;
	} 
	
	for(int nCol = 0; nCol < 3; nCol++)
	{
		pTableStatistics->adjustColumn(nCol);
	}
}

void KPGTableIndex::displayIoStatistics(QTable *pTableStatistics)
{
	QString strQuery("SELECT indexrelname, idx_blks_read, idx_blks_hit FROM pg_catalog.pg_statio_all_indexes WHERE indexrelid = ");
	strQuery.append(QString("%1;").arg(m_oid));
	
	//--- Header field names
	QHeader* hHeader = pTableStatistics->horizontalHeader();
	
	hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	try
	{
		KPGConnection *pDbConnection = connection();
		pqxx::work pqxxXaction(*pDbConnection, "refresh table statistics");
		pqxx::result pqxxResultStatistics = pqxxXaction.Exec(strQuery.latin1());
	
		if(pqxxResultStatistics.size() != 1)
		{
			kdError() << k_funcinfo "Expect one row in result !" <<  endl;
			return;
		}
		
		if(text(0) != pqxxResultStatistics[0]["indexrelname"].c_str())
		{
			kdDebug() << "KPGTable::displayStatistics(): table name not match" << endl;
			return;
		}
		
		int iRow = 0;
		
		//--- seq_scan
		pTableStatistics->setText(iRow, 0, i18n("idx_blks_read"));
		pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_blks_read"].c_str());
		pTableStatistics->setText(iRow++, 2, i18n("Total number of disc blocks read"));
		
		//--- heap_blks_hit
		pTableStatistics->setText(iRow, 0, i18n("idx_blks_hit"));
		pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_blks_hit"].c_str());
		pTableStatistics->setText(iRow++, 2, i18n("Number of buffer hits"));
	}
	catch (const std::exception &e)
	{
		kdError() << k_funcinfo << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return;
	} 
	
	for(int nCol = 0; nCol < 3; nCol++)
	{
		pTableStatistics->adjustColumn(nCol);
	}
}



