//
// C++ Interface: kpgtableindex
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef KPGTABLEINDEX_H
#define KPGTABLEINDEX_H

#include <qmap.h>
  
#include "kpgobject.h"

class KPGTableIndexesFolder;

// Map of index columns - pair of <int iColumnNum, QString strColumnName> 	
typedef QMap<int, QString> MapIndexKey;

/**
Base class for table indexes - primary key and index

@author Lumir Vanek
*/

class KPGTableIndex : public KPGObject
{
public:
    KPGTableIndex(KPGTableIndexesFolder *, const QString, pqxx::oid);
	KPGTableIndex(KPGTableIndexesFolder *, KPGTableIndex *, const QString, pqxx::oid);
    ~KPGTableIndex();
           	
	const MapIndexKey & getMapIndexKey() const { return m_mapIndexKey; }	
		
	// Get SQL object type 
  	virtual const char * getObjectType() const { return "INDEX"; }	
		
	void setProperties(const QString,	const QString, bool, bool, bool, const QString &);
    void setProperties(const QString, const QString, bool, bool, bool, pqxx::oid, const QString, const QString &);
	
	// Refresh. Behavior depends from type of object
	virtual bool refresh() { return refreshItem(); }
	
	// Refresh only this item, not childs
	virtual bool refreshItem();
	
	// Refresh map of index columns
	bool refreshMapIndexKey(pqxx::oid, pqxx::work &);
		
    bool isUnique() const { return m_bIsUnique; }
    bool isPrimary() const { return m_bIsPrimary; }
    bool isClustered() const { return m_bIsClustered; }
    const QString indKey() const { return m_strIndKey; }
    //const QString & owner() const { return m_strOwner; }
    const QString & tablespace() const { return m_strTablespace; }
    
    
protected:

    void displayStatistics(QTable *);
    void displayIoStatistics(QTable *);
    
protected:
	// Object properties
	// name is stored as text(0)
	QString m_strIndKey;
	bool m_bIsUnique;
	bool m_bIsPrimary;
	bool m_bIsClustered;
  	pqxx::oid m_oidTablespace;   // For PostgreSQL 8 or newer 
  	QString m_strTablespace;     // For PostgreSQL 8 or newer
  	QString m_strIndexdef;
  	
  	// Map of index columns - pair of <int iColumnNum, QString strColumnName> 
  	MapIndexKey m_mapIndexKey;
};

#endif
