//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtablespace.h"

// include files for Qt
#include <qtable.h>

// include files for KDE
#include <kiconloader.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgserver.h"
#include "kpgtable.h"
#include "kpgtablespacesfolder.h"
#include "../PropertyPages/kpgtablespaceproppage.h"
#include "../kpgsqldialog.h"


KPGTablespace::KPGTablespace(KPGTablespacesFolder *parent, const QString name, pqxx::oid _oid)
 : KPGObject(parent, name, _oid)
{
  if((name == "pg_default") || (name == "pg_global"))
  { 
    setPixmap(0, *m_pIconTablespaceSys);
    m_bIsSystemObject = true;
  }
  else
  {
    setPixmap(0, *m_pIconTablespaceUsr);
    m_bIsSystemObject = false;
  }
}

KPGTablespace::KPGTablespace(KPGTablespacesFolder *parent, KPGTablespace *after, const QString name, pqxx::oid _oid)
 : KPGObject(parent, after, name, _oid)
{
  if((name == "pg_default") || (name == "pg_global"))
  { 
    setPixmap(0, *m_pIconTablespaceSys);
    m_bIsSystemObject = true;
  }
  else
  {
    setPixmap(0, *m_pIconTablespaceUsr);
    m_bIsSystemObject = false;
  }
}

KPGTablespace::~KPGTablespace()
{
}

void KPGTablespace::setProperties(
	const QString strOwner,
	const QString strLocation, 
	const QString strACL
	)
{
	m_strOwner = strOwner;
	m_strLocation = strLocation;
	m_strACL = strACL;
}

void KPGTablespace::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	KPGTablespacePropPage *pPropertyPage = static_cast <KPGTablespacePropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  	QHeader* hHeader = pTableProperties->horizontalHeader();

  	hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
		
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  	pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Name of the tablespace"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  	pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the tablespace"));
	
	//--- Location
	pTableProperties->setText(iRow, 0, i18n("Location"));
  	pTableProperties->setText(iRow, 1, m_strLocation);
	pTableProperties->setText(iRow++, 2, i18n("Location of tablespace"));
	
	//--- ACL
	pPropertyPage->setACL(m_strACL);
  
	for(int nCol = 0; nCol < 3; nCol++)
	{
		pTableProperties->adjustColumn(nCol);
	}
}

// Refresh only function info, without childs objects
bool KPGTablespace::refreshItem()
{
  // Get pointer to server for version info
  KPGServer *pServer = static_cast <KPGServer *> (parent()->parent());
  
  work pqxxXaction(* pServer->connection(), "refreshItem"); 

  // obtain list of tablespaces
  QString strQuery("SELECT ts.oid, ts.spcname, sh.usename AS owner, ts.spclocation, ts.spcacl ");
  strQuery.append("FROM pg_catalog.pg_tablespace ts ");
  strQuery.append("LEFT JOIN pg_catalog.pg_shadow sh ON ts.spcowner=sh.usesysid  ");
  
  strQuery.append("WHERE ts.oid = " + QString("%1").arg(m_oid));
    
  try
  {
    pqxx::result pqxxResultTablespaces = pqxxXaction.Exec(strQuery.latin1());

    setProperties(
      pqxxResultTablespaces[0]["owner"].c_str(), 
      pqxxResultTablespaces[0]["spclocation"].c_str(),
      pqxxResultTablespaces[0]["spcacl"].c_str()
      );
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
    dlg.exec();
    return false;
  } 
  
  return true;
}

