//
// C++ Implementation: kpgtrigger
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtrigger.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpgdatabase.h"
#include "kpgtriggersfolder.h"
#include "../PropertyPages/kpgtriggerproppage.h"
#include "../kpgsqldialog.h"

KPGTrigger::KPGTrigger(KPGTriggersFolder *parent, const QString name, pqxx::oid _oid)
 : KPGObject(parent, name, _oid)
{
	setPixmap(0, *m_pIconTrigger);
}

KPGTrigger::KPGTrigger(KPGTriggersFolder *parent, KPGTrigger *after, const QString name, pqxx::oid _oid)
 : KPGObject(parent, after, name, _oid)
{
	setPixmap(0, *m_pIconTrigger);
}

KPGTrigger::~KPGTrigger()
{
}

// Refresh only column info, without childs objects
bool KPGTrigger::refreshItem()
{
	// Get pointer to owner table and database
	KPGTable *pTable = static_cast <KPGTable *> (parent()->parent());
	KPGDatabase *pDatabase = static_cast <KPGDatabase *> (pTable->parent()->parent()->parent());
			
	work pqxxXaction(* pDatabase->connection(), "refreshItem"); 
		
	QString strQuery("SELECT t.oid, t.tgname, des.description, t.tgenabled, t.tgdeferrable, t.tginitdeferred, tgnargs, tgargs, nspname, tgfoid,  pr.proname ");
				
	strQuery.append("FROM pg_catalog.pg_trigger t ");
	strQuery.append("JOIN pg_catalog.pg_class cl ON cl.oid=tgrelid ");
	strQuery.append("JOIN pg_catalog.pg_namespace na ON na.oid=relnamespace ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON des.objoid=t.oid ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_proc pr ON tgfoid=pr.oid ");
	strQuery.append("WHERE NOT tgisconstraint ");
	strQuery.append("AND tgrelid=");
	strQuery.append(QString("%1").arg(pTable->oid()));
	strQuery.append(" AND t.oid=");
	strQuery.append(QString("%1").arg(m_oid));
	
	try
	{
    	pqxx::result pqxxResultTriggers = pqxxXaction.Exec(strQuery.latin1());

		if(pqxxResultTriggers.size() != 1)
		{
			kdError() << k_funcinfo "Expect one row in result !" <<  endl;
			return false;
		}
		
		bool bIsEnabled;
		bool bDeferrable;
		bool bInitDeferred;
		int iNArgs;
		pqxx::oid oidProcedure = 0;
				
		pqxxResultTriggers[0]["tgenabled"].to(bIsEnabled);
		pqxxResultTriggers[0]["tgdeferrable"].to(bDeferrable);
		pqxxResultTriggers[0]["tginitdeferred"].to(bInitDeferred);
		pqxxResultTriggers[0]["tgnargs"].to(iNArgs);
		pqxxResultTriggers[0]["tgfoid"].to(oidProcedure);
					
		setProperties(
			pqxxResultTriggers[0]["description"].c_str(),
			bIsEnabled,
			bDeferrable,
			bInitDeferred,
			iNArgs,
			pqxxResultTriggers[0]["tgargs"].c_str(),
			pqxxResultTriggers[0]["nspname"].c_str(),
			oidProcedure,
			pqxxResultTriggers[0]["proname"].c_str()
			);
		
	}
	catch (const std::exception &e)
	{
		kdError() << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return false;
	} 
	
	return true;
}


void KPGTrigger::setProperties(
		const QString strDescription,
		bool bIsEnabled,
	  bool bDeferrable,
	  bool bInitDeferred,
	  int iNArgs,
		const QString strArgs,
	  const QString strNameSpace,
		pqxx::oid oidProcedure,
		const QString strProcName
	)
{
	m_strDescription = strDescription;
	m_bIsEnabled = bIsEnabled;
	m_bDeferrable = bDeferrable;
	m_bInitDeferred = bInitDeferred;
	m_iNArgs = iNArgs;
	m_strArgs = strArgs;
	m_strNameSpace = strNameSpace;
	m_oidProcedure = oidProcedure;
	m_strProcName = strProcName;
}

void KPGTrigger::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	// Display server version info
	KPGTriggerPropPage *pPropertyPage = static_cast <KPGTriggerPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  	QHeader* hHeader = pTableProperties->horizontalHeader();

  	hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  	pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Trigger name (must be unique among triggers of same table)"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  	pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the trigger"));
		
	//--- Is Enabled
	pTableProperties->setText(iRow, 0, i18n("Is Enabled"));
  	pTableProperties->setText(iRow, 1, (m_bIsEnabled) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if trigger is enabled (not presently checked everywhere it should be, so disabling a trigger by setting this false does not work reliably)"));
	
	//--- Is Deferrable
	pTableProperties->setText(iRow, 0, i18n("Is Deferrable"));
  	pTableProperties->setText(iRow, 1, (m_bDeferrable) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if deferrable"));
	
	//--- Is Is initially Deferred
	pTableProperties->setText(iRow, 0, i18n("Is initially Deferred"));
 	pTableProperties->setText(iRow, 1, (m_bInitDeferred) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if initially deferred"));
	
	//--- Number of argument
	pTableProperties->setText(iRow, 0, i18n("Number of arguments"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_iNArgs));
	pTableProperties->setText(iRow++, 2, i18n("Number of argument strings passed to trigger function"));
	
	//--- Arguments
	pTableProperties->setText(iRow, 0, i18n("Arguments"));
  	pTableProperties->setText(iRow, 1, m_strArgs);
	pTableProperties->setText(iRow++, 2, i18n("Argument strings to pass to trigger, each null-terminated"));
	
	//--- Namespace
	pTableProperties->setText(iRow, 0, i18n("Namespace"));
  	pTableProperties->setText(iRow, 1, m_strNameSpace);
	pTableProperties->setText(iRow++, 2, i18n("Name of the namespace"));
	
	//--- Procedure name
	pTableProperties->setText(iRow, 0, i18n("Procedure name"));
  	KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Procedure name" );
	pLabel->setValues(m_oidProcedure, m_strProcName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Name of the trigger procedure, to be called"));
	
	for(int nCol = 0; nCol < 3; nCol++)
	{
		pTableProperties->adjustColumn(nCol);
	}
}


