/***************************************************************************
                          kpgview.cpp  -  description
                             -------------------
    begin                : Po led 19 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpgview.h"
 
// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktextedit.h>

#include <qtable.h>

#include "kpgdatabase.h"
#include "kpgviewsfolder.h"
#include "../PropertyPages/kpgviewproppage.h"
#include "../kpgsqldialog.h"


KPGView::KPGView(KPGViewsFolder *parent, const QString name, pqxx::oid _oid)
 : KPGObject(parent, name, _oid)
{
	setPixmap(0, *m_pIconView);
}

KPGView::KPGView(KPGViewsFolder *parent, KPGView *after, const QString name, pqxx::oid _oid)
 : KPGObject(parent, after, name, _oid)
{
	setPixmap(0, *m_pIconView);
}

KPGView::~KPGView()
{
}

void KPGView::setProperties(
	const QString strOwner, 
	const QString strACL,
	const QString strDescription
	)
{
	m_strOwner = strOwner;
	m_strACL = strACL;
	m_strDescription = strDescription;
}


void KPGView::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	KPGViewPropPage *pPropertyPage = static_cast <KPGViewPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  	QHeader* hHeader = pTableProperties->horizontalHeader();

  	hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  	pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  	pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Name of the view"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the view"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  	pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the view"));
	
	//--- ACL
	pPropertyPage->setACL(m_strACL);
				
	for(int nCol = 0; nCol < 3; nCol++)
	{
		pTableProperties->adjustColumn(nCol);
	}
	
	pPropertyPage->m_pTextEditDefinition->setText(m_strDefinition);
}

// Refresh only view info, without childs objects
bool KPGView::refreshItem()
{
	// Get pointer to server for version info
	KPGDatabase *pDatabase = static_cast <KPGDatabase *> (parent()->parent()->parent());
	
	work pqxxXaction(* pDatabase->connection(), "refreshItem"); 
	
	// obtain info about of view
	QString strQuery("SELECT c.oid, c.relname, description, pg_get_userbyid(c.relowner) AS viewowner, c.relacl ");
	strQuery.append("FROM pg_catalog.pg_class c ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON (des.objoid=c.oid and des.objsubid=0) ");
	
	strQuery.append("WHERE c.oid = " + QString("%1").arg(m_oid));
		
	try
	{
		pqxx::result pqxxResultViews = pqxxXaction.Exec(strQuery.latin1());
	
		if(pqxxResultViews.size() != 1)
		{
			kdError() << k_funcinfo "Expect one row in result !" <<  endl;
			return false;
		}
		
		setProperties(
		pqxxResultViews[0]["viewowner"].c_str(), 
		pqxxResultViews[0]["relacl"].c_str(),
		pqxxResultViews[0]["description"].c_str()
		);
			
		//-- Get definition
		strQuery = QString("SELECT pg_get_viewdef(%1) AS definition;").arg(m_oid);
		pqxx::result pqxxResultDefinition = pqxxXaction.exec(strQuery.latin1());
		
		if (pqxxResultDefinition.size() > 0)
		{ 
		setDefinition(pqxxResultDefinition[0][0].c_str());
		} 
	}
	catch (const std::exception &e)
	{
		kdError() << e.what() << endl;
		KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
		return false;
	} 
	
	return true;
}

