/***************************************************************************
                          kpgviewsfolder  -  description
                             -------------------
    begin                : Ne no 1 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "kpgviewsfolder.h"

#include <qtable.h>

#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgconnection.h"
#include "kpgschema.h"
#include "kpgview.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "../kpgsqldialog.h"

KPGViewsFolder::KPGViewsFolder(KPGSchema *parent, pqxx::oid oidNameSpace)
 : KPGItemsFolder(parent, i18n("Views"))
{
	setPixmap(0, *m_pIconFolderGreen);
	m_oidNameSpace = oidNameSpace;
}


KPGViewsFolder::~KPGViewsFolder()
{
}

bool KPGViewsFolder::refresh(pqxx::work &pqxxXaction)
{
    // delete all child items (databases)
    while (QListViewItem * pItem = firstChild())
        delete pItem;

	// obtain list of views
    QString strQuery("SELECT c.oid, c.relname, description, pg_get_userbyid(c.relowner) AS viewowner, c.relacl ");
	strQuery.append("FROM pg_catalog.pg_class c ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON (des.objoid=c.oid and des.objsubid=0) ");
	strQuery.append("WHERE ((c.relhasrules AND (EXISTS (SELECT r.rulename FROM pg_catalog.pg_rewrite r WHERE ((r.ev_class = c.oid) AND (bpchar(r.ev_type) = '1'::bpchar)) ))) OR \
(c.relkind = 'v'::char)) ");
	strQuery.append("AND relnamespace = " + QString("%1").arg(m_oidNameSpace) + " ");
	strQuery.append("ORDER BY relname");
		
  try
  {
    m_pqxxResultViews = pqxxXaction.Exec(strQuery.latin1());

		KPGView *pgView = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultViews.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultViews[i][0].to(oid);
		
			if(pgView == 0)
      	pgView = new KPGView(this, m_pqxxResultViews[i]["relname"].c_str(), oid);
			else
				pgView = new KPGView(this, pgView, m_pqxxResultViews[i]["relname"].c_str(), oid);
						
			pgView->setProperties(
				m_pqxxResultViews[i]["viewowner"].c_str(), 
				m_pqxxResultViews[i]["relacl"].c_str(),
				m_pqxxResultViews[i]["description"].c_str()
				);
				
			//-- Get definition
			strQuery = QString("SELECT pg_get_viewdef(%1) AS definition;").arg(oid);
			pqxx::result pqxxResultDefinition = pqxxXaction.exec(strQuery.latin1());
		
			if (pqxxResultDefinition.size() > 0)
    	{ 
				pgView->setDefinition(pqxxResultDefinition[0][0].c_str());
			}	
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGViewsFolder::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  	pPropertyPage->setLabelText(i18n("Views &list:"));
	
	//----------------------------------------------------------------------------
	//Display views list
		
	QTable *pTableViews = pPropertyPage->m_pTableItems;
  
 // fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultViews, pTableViews);

  // field names
  QHeader* hHeader = pTableViews->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Owner"));
  hHeader->setLabel(nCol++, i18n("ACL"));

  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableViews->adjustColumn(nCol);
  }
}



