//
// C++ Implementation: kpgfunctionproppage
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgfunctionproppage.h"

// include files for Qt
#include <qiconset.h>
#include <qtable.h> 

// include files for KDE
#include <ktabwidget.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <klistview.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <ktextedit.h>
#include <kdebug.h>

// application specific includes
#include "../QueryResult/kpgsyntaxhighlighter.h"
#include "../kpogreview.h"
#include "../kpgconfiguration.h"
#include "../QueryResult/kpgsqleditorsettings.h"
#include "../DbObjects/kpgschema.h"
#include "../DbObjects/kpgtablecolumn.h"
#include "../kpglinklabel.h"
#include "../kpgutil.h"


KPGFunctionPropPage::KPGFunctionPropPage(QWidget *parent, KPGFunction *pFunction)
 : KPGFunctionPropPageBase(parent, "KPGFunctionPropPage")
{
	m_pFunction = pFunction;
    m_pSyntaxHighlighter = new KPGSyntaxHighlighter(m_pTextEditSource);
	
	// Apply current configuration
	slotSqlEditorSettingsChanged();
 	// and make sure to be informed about its changes.
	connect( KPoGreView::configuration()->sqleditor(), SIGNAL(sigChanged()), this, SLOT(slotSqlEditorSettingsChanged()) );
  
    m_pTabWidget->setTabIconSet(tabACL, QIconSet(QPixmap(UserIcon("group.png"))));
    m_pTabWidget->setTabIconSet(tabArguments, QIconSet(QPixmap(UserIcon("argument.png"))));
    
    m_pListViewArguments->setSortColumn(-1);
}


KPGFunctionPropPage::~KPGFunctionPropPage()
{
	disconnect( KPoGreView::configuration()->sqleditor(), SIGNAL(sigChanged()), this, SLOT(slotSqlEditorSettingsChanged()) );
  
  delete m_pSyntaxHighlighter;
}

void KPGFunctionPropPage::slotSqlEditorSettingsChanged()
{
    m_pTextEditSource->setFont( KPoGreView::configuration()->sqleditor()->font() );
    
	m_pSyntaxHighlighter->setColorDefaultText( KPoGreView::configuration()->sqleditor()->colorDefaultText() );
	m_pSyntaxHighlighter->setColorKeyWords( KPoGreView::configuration()->sqleditor()->colorKeyWords() );
	m_pSyntaxHighlighter->setColorDataTypes( KPoGreView::configuration()->sqleditor()->colorDataTypes() );
	m_pSyntaxHighlighter->setColorOperators( KPoGreView::configuration()->sqleditor()->colorOperators() );
	m_pSyntaxHighlighter->setColorQuotedStrings( KPoGreView::configuration()->sqleditor()->colorQuotedStrings() );
	m_pSyntaxHighlighter->setColorNumbers( KPoGreView::configuration()->sqleditor()->colorNumbers() );
    m_pSyntaxHighlighter->setColorComments( KPoGreView::configuration()->sqleditor()->colorComments() );
	
	m_pSyntaxHighlighter->rehighlight();
}

void KPGFunctionPropPage::slotTablePropDblClicked(int iRow, int iCol, int /*iButton*/, const QPoint & /*ptMousePos*/)
{
    QWidget *w = m_pTableProperties->cellWidget(iRow, iCol);
	if(w == 0)
		return;
					
	KPGLinkLabel *pLabel = static_cast <KPGLinkLabel *> (w);
	pqxx::oid _oid = pLabel->oid();
			
	if(_oid != 0)
		emit sigSearchObject(_oid);
}

void KPGFunctionPropPage::setACL(const QString &strACL)
{
	m_pListViewACL->clear();
	m_listOfAclItems.clear();
	m_pLineEditACL->setText(strACL);
	
	QString strAclItem; // string for one ACL item, e.g. miriam=arwdRxt/miriam
	for(unsigned int i = 0; i < strACL.length(); i++)
	{
		if(strACL[i] == '{')
		continue;
		
		if((strACL[i] == ',') || (strACL[i] == ','))
		{
			m_listOfAclItems.append(KPGAclItem(strAclItem)); // create new ACL item
			strAclItem.truncate(0); // clear ACL string
		}
		else
		{
			strAclItem.append(strACL[i]);
		}
	}
	
	if(!strAclItem.isEmpty())
		m_listOfAclItems.append(KPGAclItem(strAclItem)); // create new ACL item
	
	// Traverse list of ACL items
	KPGAclItemList::iterator it;
	for ( it = m_listOfAclItems.begin(); it != m_listOfAclItems.end(); ++it )
	{   
		QListViewItem * pItem = new QListViewItem( m_pListViewACL, 0 );
		(*it).setListViewItem(pItem);
		
		pItem->setText(0, (*it).grantee());
		pItem->setText(1, (*it).grantor());
		
		pItem->setPixmap(2, UserIcon((*it).canExecute() ? "box_checked.png" : "box_clear.png"));
		
		if((*it).canPassGrantToOther())
		pItem->setPixmap(3, UserIcon((*it).canGrantExecute() ? "box_checked.png" : "box_clear.png"));
	} 
}

void  KPGFunctionPropPage::setArgumentsList(const KPGFunction::ListArguments & listArguments)
{
	m_pListViewArguments->clear();
	
	for(KPGFunction::ListArguments::const_iterator cit = listArguments.begin(); cit != listArguments.end(); ++cit)
	{ 
		
		QListViewItem * pItem = new QListViewItem( m_pListViewArguments, 0);
		
		if((*cit).typName().length() > 0)
		{
			QPixmap *pTypePixmap = KPGTableColumn::getPixmapForType((*cit).typName());
			pItem->setPixmap(0, *pTypePixmap);
		}
		
		pItem->setText(0, (*cit).typName());
		pItem->setText(1, (*cit).name());
		
		if((*cit).mode() == "i") pItem->setText(2, "IN");
		else if((*cit).mode() == "o") pItem->setText(2, "OUT");
		else if((*cit).mode() == "b") pItem->setText(2, "INOUT");
    }
}

void KPGFunctionPropPage::slotAclListViewClicked(QListViewItem *pItem, const QPoint &, int iColumn)
{
  // Find KPGAclItem using pItem value
  KPGAclItemList::iterator it;
  for ( it = m_listOfAclItems.begin(); it != m_listOfAclItems.end(); ++it )
  {   
    if(pItem == (*it).getListViewItem())
    {
      // found ...
      
      switch(iColumn)
      {
        case 0:
        case 1: // do nothing on first 2 columns
            break;
        
        case 2:
            pItem->setPixmap(2, UserIcon((*it).toggleExecute() ? "box_checked.png" : "box_clear.png"));
            break;
       
        case 3:
            if((*it).canPassGrantToOther())
              pItem->setPixmap(3, UserIcon((*it).toggleGrantExecute() ? "box_checked.png" : "box_clear.png"));
            
            break;
            
        default:
                kdError() << k_funcinfo << " Unexpected column !" << endl;
                break;      
      }
    
      m_pPushButtonUpdateACL->setEnabled(true);
      return;
    }
  }
}

void KPGFunctionPropPage::slotUpdateACL()
{
  QString strSQL;
  
  KPGSchema *pSchema = static_cast <KPGSchema *> (m_pFunction->parent()->parent());
    
  // Traverse all items and get their SQL's
  KPGAclItemList::iterator it;
  for ( it = m_listOfAclItems.begin(); it != m_listOfAclItems.end(); ++it )
  {   
    strSQL.append((*it).getSQL("FUNCTION " + KPGUtil::fullyQualifiedName(pSchema->text(0), m_pFunction->text(0)) + m_pFunction->argumentTypes()));
  }
  
  if(!strSQL.isEmpty())
    emit sigRunWizard(strSQL);
}


// Consumes request for context menu for property table
void KPGFunctionPropPage::slotTablePropContextMenuRequested(int row, int col, const QPoint & pos)
{
	emit sigTablePropContextMenuRequested(row, col, pos);	
}

#include "kpgfunctionproppage.moc"
