//
// C++ Implementation: kpgindexproppage
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgindexproppage.h"

// include files for Qt
#include <qiconset.h>
#include <qtable.h>  

// include files for KDE
#include <ktabwidget.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <kdebug.h>

#include "../kpglinklabel.h"
#include "../QueryResult/kpgsyntaxhighlighter.h"
#include "../kpogreview.h"
#include "../kpgconfiguration.h"
#include "../QueryResult/kpgsqleditorsettings.h"
#include "../DbObjects/kpgindex.h"

KPGIndexPropPage::KPGIndexPropPage(QWidget *parent, KPGIndex *pIndex)
 : KPGIndexPropPageBase(parent, "KPGIndexPropPageBase")
{
  m_pIndex = pIndex;
  m_pSyntaxHighlighter = new KPGSyntaxHighlighter(m_pTextEditDefinition);
  
  // Apply current configuration
  slotSqlEditorSettingsChanged();
  // and make sure to be informed about its changes.
  connect( KPoGreView::configuration()->sqleditor(), SIGNAL(sigChanged()), this, SLOT(slotSqlEditorSettingsChanged()) );
  
  m_pTabWidget->setTabIconSet(tabStatistics, QIconSet(QPixmap(SmallIcon("ksysguard.png")))); 
  m_pButtonRefresh->setIconSet(QIconSet(QPixmap(BarIcon("reload.png")))); 
}

KPGIndexPropPage::~KPGIndexPropPage()
{
}

void KPGIndexPropPage::slotSqlEditorSettingsChanged()
{
  m_pTextEditDefinition->setFont( KPoGreView::configuration()->sqleditor()->font() );
  
  m_pSyntaxHighlighter->setColorDefaultText( KPoGreView::configuration()->sqleditor()->colorDefaultText() );
  m_pSyntaxHighlighter->setColorKeyWords( KPoGreView::configuration()->sqleditor()->colorKeyWords() );
  m_pSyntaxHighlighter->setColorDataTypes( KPoGreView::configuration()->sqleditor()->colorDataTypes() );
  m_pSyntaxHighlighter->setColorOperators( KPoGreView::configuration()->sqleditor()->colorOperators() );
  m_pSyntaxHighlighter->setColorQuotedStrings( KPoGreView::configuration()->sqleditor()->colorQuotedStrings() );
  m_pSyntaxHighlighter->setColorNumbers( KPoGreView::configuration()->sqleditor()->colorNumbers() );
  m_pSyntaxHighlighter->setColorComments( KPoGreView::configuration()->sqleditor()->colorComments() );
  
  m_pSyntaxHighlighter->rehighlight();
}

void KPGIndexPropPage::slotTablePropDblClicked(int iRow, int iCol, int /*iButton*/, const QPoint & /*ptMousePos*/)
{
    QWidget *w = m_pTableProperties->cellWidget(iRow, iCol);
		if(w == 0)
			return;
					
		KPGLinkLabel *pLabel = static_cast <KPGLinkLabel *> (w);
		pqxx::oid _oid = pLabel->oid();
			
		if(_oid != 0)
			emit sigSearchObject(_oid);
}

void KPGIndexPropPage::slotRefreshStatistics()
{
  m_pIndex->displayStatistics(this);
  m_pIndex->displayIoStatistics(this);
}

// Consumes request for context menu for property table
void KPGIndexPropPage::slotTablePropContextMenuRequested(int row, int col, const QPoint & pos)
{
	emit sigTablePropContextMenuRequested(row, col, pos);	
}

#include "kpgindexproppage.moc"
