//
// C++ Implementation: kpgpropertypagechildview
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgpropertypagechildview.h"

#include <qlayout.h>

#include <kiconloader.h>
#include <kdebug.h>

#include "../kpogreview.h"

// Database objects includes
#include "../DbObjects/kpgserver.h"
#include "../DbObjects/kpgdatabase.h"
#include "../DbObjects/kpgschema.h"
#include "../DbObjects/kpgview.h"
#include "../DbObjects/kpgtable.h"
#include "../DbObjects/kpgsequence.h"
#include "../DbObjects/kpgfunction.h"
#include "../DbObjects/kpgprimarykey.h"
#include "../DbObjects/kpgindex.h"
#include "../DbObjects/kpglanguage.h"

// Property pages includes
#include "kpgvoidproppage.h"
#include "kpgpropertypage.h"
#include "kpgserverproppage.h"
#include "kpgtablespaceproppage.h"
#include "kpgdatabaseproppage.h"
#include "kpgschemaproppage.h"
#include "kpgtableproppage.h"
#include "kpgviewproppage.h"
#include "kpgtablecolumnproppage.h"
#include "kpgprimarykeyconstrproppage.h"
#include "kpgforeignkeyconstrproppage.h"
#include "kpgcheckconstrproppage.h"
#include "kpguniqueconstrproppage.h"
#include "kpgprimarykeyproppage.h"
#include "kpgindexproppage.h"
#include "kpgruleproppage.h"
#include "kpgtriggerproppage.h"
#include "kpgfunctionproppage.h"
#include "kpgsequenceproppage.h"
#include "kpgoperatorproppage.h"
#include "kpgoperatorclassproppage.h"
#include "kpgaggregateproppage.h"
#include "kpguserproppage.h"
#include "kpggroupproppage.h"
#include "kpgtypeproppage.h"
#include "kpglanguageproppage.h"
#include "kpgdomainproppage.h"
#include "kpgconversionproppage.h"
#include "kpgcastproppage.h"
#include "kpgdomainconstrproppage.h"

KPGPropertyPageChildView::KPGPropertyPageChildView(KPoGreView *pPoGreView)
 : KMdiChildView(i18n("Properties"), pPoGreView, "KPGPropertyPageChildView")
{
	m_pPoGreView = pPoGreView;
	
	m_pWidgetProperties = 0;
	m_pPropertyPage = 0;
	
	m_pLayout = new QHBoxLayout(this, 0, -1, "layout" );
	
	setIcon( SmallIcon("history") );
}


KPGPropertyPageChildView::~KPGPropertyPageChildView()
{
	destroyPropertiesWidget();
}

// Destroy Properties widget
void KPGPropertyPageChildView::destroyPropertiesWidget()
{
	// Close old widget property page
	if(m_pWidgetProperties)
	{
		// disconnect all
		disconnect(m_pWidgetProperties, 0, 0, 0);
		m_pWidgetProperties->close(true);
		m_pWidgetProperties = 0;
	}
  
}

// Create Properties widget, with regards to selected type in tree
QWidget* KPGPropertyPageChildView::createPropertiesWidget(KPGTreeItem * pItem)
{
	destroyPropertiesWidget();
	
	if(pItem == 0)
		return 0;
		
	m_pWidgetProperties = 0;
    m_pPropertyPage = 0;
	
  	// create new property page
  	switch(pItem->type())
    {
		//******************************************************************************************
		//
		// Main objects
		//
		//******************************************************************************************
				
        case KPGTreeItem::nodeServer:
        {
			KPGServerPropPage *pPropPage = new KPGServerPropPage(this, static_cast <KPGServer *> (pItem));  
			m_pWidgetProperties = pPropPage;
			m_pPropertyPage = pPropPage;
		}	
			break;
		
		
        case KPGTreeItem::nodeDatabase:
		{
			KPGDatabasePropPage *pPropPage = new KPGDatabasePropPage(this, static_cast <KPGDatabase *> (pItem));
			m_pWidgetProperties = pPropPage;
			m_pPropertyPage = pPropPage;
          
          connect(pPropPage, SIGNAL(sigRunWizard(const QString &)), m_pPoGreView, SLOT( slotRunWizard(const QString &))); 
        }
          break;
        				
		case KPGTreeItem::nodeSchema:
        {
          	KPGSchemaPropPage *pPropPage = new KPGSchemaPropPage(this, static_cast <KPGSchema *> (pItem));
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
          			
          	connect(pPropPage, SIGNAL(sigCurrentTabChanged(KPGSchemaPropPage *, QWidget *)), m_pPoGreView, SLOT(slotSchemaPropTabChanged(KPGSchemaPropPage *, QWidget *)));
          
          	connect(pPropPage, SIGNAL(sigRunWizard(const QString &)), m_pPoGreView, SLOT( slotRunWizard(const QString &))); 
        }  
          	break;
				
		case KPGTreeItem::nodeFolderGroups:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
        }
          	break;
				
		case KPGTreeItem::nodeFolderUsers:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage( this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeFolderTablespaces:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage( this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
				
		case KPGTreeItem::nodeUser:
		{
			KPGUserPropPage *pPropPage = new KPGUserPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeGroup:
		{
			KPGGroupPropPage *pPropPage = new KPGGroupPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeTablespace:
		{
			KPGTablespacePropPage *pPropPage = new KPGTablespacePropPage(this, pItem->text(0));
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
         	
			connect(pPropPage, SIGNAL(sigRunWizard(const QString &)), m_pPoGreView, SLOT( slotRunWizard(const QString &))); 
		}
			break;
									
		//******************************************************************************************
		//
		// Database objects folders
		//
		//******************************************************************************************
										
		case KPGTreeItem::nodeFolderTables:
        {
          	KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
          	m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
        }
          	break;
				
		case KPGTreeItem::nodeFolderViews:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeFolderSequences:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeFolderFunctions:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeFolderOperators:
        {
        	KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
        	m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
        }
        	break;
					
		case KPGTreeItem::nodeFolderOperatorClasses:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeFolderAggregates:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
											
		case KPGTreeItem::nodeFolderTypes:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeFolderLanguages: 
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
				
		case KPGTreeItem::nodeFolderDomains:
        {
        	KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
        	m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
        }
          	break;
					
		case KPGTreeItem::nodeFolderConversions:
		{	KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}	
			break;
					
		case KPGTreeItem::nodeFolderCasts:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;		
				
				
		//******************************************************************************************
		//
		// Database objects
		//
		//******************************************************************************************
				
		case KPGTreeItem::nodeTable:
		{
			KPGTablePropPage *pPropPage = new KPGTablePropPage(this, static_cast <KPGTable *> (pItem));
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
			
			connect(pPropPage, SIGNAL(sigRunWizard(const QString &)), m_pPoGreView, SLOT( slotRunWizard(const QString &))); 
		}
			break;

        case KPGTreeItem::nodeView:
		{
			KPGViewPropPage *pPropPage = new KPGViewPropPage(this, static_cast <KPGView *> (pItem));
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
			
			connect(m_pWidgetProperties, SIGNAL(sigRunWizard(const QString &)), m_pPoGreView, SLOT( slotRunWizard(const QString &))); 
		}
			break;
					
		case KPGTreeItem::nodeSequence:
		{
			KPGSequencePropPage *pPropPage = new KPGSequencePropPage(this, static_cast <KPGSequence *> (pItem));
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
			
			connect(pPropPage, SIGNAL(sigRunWizard(const QString &)), m_pPoGreView, SLOT( slotRunWizard(const QString &))); 
        }
        	break;
				
		case KPGTreeItem::nodeFunction:
		{
			KPGFunctionPropPage *pPropPage = new KPGFunctionPropPage(this, static_cast <KPGFunction *> (pItem));
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
			
			connect(pPropPage, SIGNAL(sigRunWizard(const QString &)), m_pPoGreView, SLOT( slotRunWizard(const QString &))); 
		}
			break;
					
		case KPGTreeItem::nodeOperator:
        {
        	KPGOperatorPropPage *pPropPage = new KPGOperatorPropPage(this);
        	m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
        } 	
          	break;
				
		case KPGTreeItem::nodeOperatorClass:
		{
			KPGOperatorClassPropPage *pPropPage = new KPGOperatorClassPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeAggregate:
		{
			KPGAggregatePropPage *pPropPage = new KPGAggregatePropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
				
		case KPGTreeItem::nodeType:
		{
			KPGTypePropPage *pPropPage = new KPGTypePropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
        }
          	break;
				
		case KPGTreeItem::nodeLanguage:
		{
			KPGLanguagePropPage *pPropPage = new KPGLanguagePropPage( this, static_cast <KPGLanguage *> (pItem));
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
			
          	connect(pPropPage, SIGNAL(sigRunWizard(const QString &)), m_pPoGreView, SLOT( slotRunWizard(const QString &))); 
        }
          	break; 
				
		case KPGTreeItem::nodeDomain:
		{
			KPGDomainPropPage *pPropPage = new KPGDomainPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
        }
          	break; 
				
		case KPGTreeItem::nodeConversion:
		{
			KPGConversionPropPage *pPropPage = new KPGConversionPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
        }
          	break; 
				
		case KPGTreeItem::nodeCast:
		{
			KPGCastPropPage *pPropPage = new KPGCastPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
       	}
          	break;			
					
		//******************************************************************************************
		//
		// Table objects folders
		//
		//******************************************************************************************
				
						
		case KPGTreeItem::nodeFolderTableColumns:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeFolderTableConstraints:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeFolderTableIndexes:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeFolderRules:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeFolderTriggers:
        {
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
        }
          break;
        
        // Domain objects folders
            
        case KPGTreeItem::nodeFolderDomainConstraints:
		{
			KPGPropertyPage *pPropPage = new KPGPropertyPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		//******************************************************************************************
		//
		// Table objects
		//
		//******************************************************************************************
									
		case KPGTreeItem::nodeTableColumn:
		{
			KPGTableColumnPropPage *pPropPage = new KPGTableColumnPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodePrimaryKeyConstr:
		{
			KPGPrimaryKeyConstrPropPage *pPropPage = new KPGPrimaryKeyConstrPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeForeignKeyConstr:
		{
			KPGForeignKeyConstrPropPage *pPropPage = new KPGForeignKeyConstrPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeUniqueConstr:
		{
			KPGUniqueConstrPropPage *pPropPage = new KPGUniqueConstrPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeCheckConstr:
		{
			KPGCheckConstrPropPage *pPropPage = new KPGCheckConstrPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodePrimaryKey:
		{
			KPGPrimaryKeyPropPage *pPropPage = new KPGPrimaryKeyPropPage( this, static_cast <KPGPrimaryKey *> (pItem));
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeIndex:
		{
			KPGIndexPropPage *pPropPage = new KPGIndexPropPage( this, static_cast <KPGIndex *> (pItem));
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeRule:
		{
			KPGRulePropPage *pPropPage = new KPGRulePropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		case KPGTreeItem::nodeTrigger:
		{
			KPGTriggerPropPage *pPropPage = new KPGTriggerPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
          
        // Domain object
        case KPGTreeItem::nodeDomainConstr:
		{
			KPGDomainConstrPropPage *pPropPage = new KPGDomainConstrPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
					
		//******************************************************************************************
		//
		// Unselected or unknown
		//
		//******************************************************************************************
					

        case KPGTreeItem::nodeUnselected:
        	return 0;
          	break;
					
		default:
		{
			KPGVoidPropPage *pPropPage = new KPGVoidPropPage(this);
			m_pWidgetProperties = pPropPage;
         	m_pPropertyPage = pPropPage;
		}
			break;
      } 
  
  	if(m_pWidgetProperties == 0) kdError() << k_funcinfo << "m_pWidgetProperties == 0" << endl;
    if(m_pPropertyPage == 0) kdError() << k_funcinfo << "m_pPropertyPage == 0" << endl;
    
	if(m_pWidgetProperties)
	{ 
		m_pLayout->addWidget( m_pWidgetProperties );
		m_pWidgetProperties->show();
				
		connect(m_pWidgetProperties, SIGNAL(sigSearchObject(const pqxx::oid)), m_pPoGreView, SLOT( slotSearchObject(const pqxx::oid)));
		
		connect(m_pWidgetProperties, SIGNAL(sigTablePropContextMenuRequested(int, int, const QPoint &)), this, SLOT(slotTablePropContextMenuRequested(int, int, const QPoint &)));
	}

	return m_pWidgetProperties;
}

bool KPGPropertyPageChildView::requestClose()
{
	if(KMessageBox::questionYesNo(this, i18n("Really close property window ?")) == KMessageBox::Yes) 
	{
		return true;
	}
	return false;
}

void KPGPropertyPageChildView::slotTablePropContextMenuRequested(int row, int col, const QPoint & pos)
{
	emit sigTablePropContextMenuRequested(row, col, pos);
}

