//
// C++ Implementation: kpgtablespaceproppage
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtablespaceproppage.h"

// include files for Qt
#include <qiconset.h>
#include <qtable.h> 

// include files for KDE
#include <ktabwidget.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <klistview.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <kdebug.h>


KPGTablespacePropPage::KPGTablespacePropPage(QWidget *parent, const QString &strTablespace)
 : KPGTablespacePropPageBase(parent, "KPGTablespacePropPage")
{
  m_strTablespace = strTablespace;
  m_pTabWidget->setTabIconSet(tabACL, QIconSet(QPixmap(UserIcon("group.png"))));
}


KPGTablespacePropPage::~KPGTablespacePropPage()
{
}

void KPGTablespacePropPage::setACL(const QString &strACL)
{
  m_pListViewACL->clear();
  m_listOfAclItems.clear();
  m_pLineEditACL->setText(strACL);
  
  QString strAclItem; // string for one ACL item, e.g. miriam=arwdRxt/miriam
  for(unsigned int i = 0; i < strACL.length(); i++)
  {
    if(strACL[i] == '{')
      continue;
      
    if((strACL[i] == ',') || (strACL[i] == ','))
    {
      m_listOfAclItems.append(KPGAclItem(strAclItem)); // create new ACL item
      strAclItem.truncate(0); // clear ACL string
    }
    else
    {
      strAclItem.append(strACL[i]);
    }
  }
  
  if(!strAclItem.isEmpty())
    m_listOfAclItems.append(KPGAclItem(strAclItem)); // create new ACL item
  
  // Traverse list of ACL items
  KPGAclItemList::iterator it;
  for ( it = m_listOfAclItems.begin(); it != m_listOfAclItems.end(); ++it )
  {   
    QListViewItem * pItem = new QListViewItem( m_pListViewACL, 0 );
    (*it).setListViewItem(pItem);
    
    pItem->setText( 0, (*it).grantee() );
    pItem->setText( 1, (*it).grantor() );
    
    pItem->setPixmap(2, UserIcon((*it).canCreate() ? "box_checked.png" : "box_clear.png"));
    
    if((*it).canPassGrantToOther())
      pItem->setPixmap(3, UserIcon((*it).canGrantCreate() ? "box_checked.png" : "box_clear.png"));
  } 
}

void KPGTablespacePropPage::slotAclListViewClicked(QListViewItem *pItem, const QPoint &, int iColumn)
{
  // Find KPGAclItem using pItem value
  KPGAclItemList::iterator it;
  for ( it = m_listOfAclItems.begin(); it != m_listOfAclItems.end(); ++it )
  {   
    if(pItem == (*it).getListViewItem())
    {
      // found ...
      
      switch(iColumn)
      {
        case 0:
        case 1: // do nothing on first 2 columns
            break;
        
        case 2:
            pItem->setPixmap(2, UserIcon((*it).toggleCreate() ? "box_checked.png" : "box_clear.png"));
            break;
       
        case 3:
            if((*it).canPassGrantToOther())
              pItem->setPixmap(3, UserIcon((*it).toggleGrantCreate() ? "box_checked.png" : "box_clear.png"));
            
            break;
            
        default:
                kdError() << k_funcinfo << " Unexpected column !" << endl;
                break;      
      }
    
      m_pPushButtonUpdateACL->setEnabled(true);
      return;
    }
  }
}

void KPGTablespacePropPage::slotUpdateACL()
{
  QString strSQL;
  
  // Traverse all items and get their SQL's
  KPGAclItemList::iterator it;
  for ( it = m_listOfAclItems.begin(); it != m_listOfAclItems.end(); ++it )
  {   
    strSQL.append((*it).getSQL("TABLESPACE " + m_strTablespace));
  }
  
  if(!strSQL.isEmpty())
    emit sigRunWizard(strSQL);
}

// Consumes request for context menu for property table
void KPGTablespacePropPage::slotTablePropContextMenuRequested(int row, int col, const QPoint & pos)
{
	emit sigTablePropContextMenuRequested(row, col, pos);	
}

#include "kpgtablespaceproppage.moc"
