//
// C++ Implementation: kpgqueryresultchildview
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgqueryresultchildview.h"

// include files for Qt
#include <qdir.h>
#include <qlayout.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <kiconloader.h>
#include <ktextedit.h>

// application specific includes
#include "kpgsqledit.h"
#include "kpgqueryresultwindow.h"
#include "../kpogreview.h"

KPGQueryResultChildView::KPGQueryResultChildView(KPoGreView *pPoGreView)
 : KMdiChildView(i18n("Untitled"), pPoGreView, "KPGQueryResultChildView")
{
	m_pPoGreView = pPoGreView;
	
	QBoxLayout* pLayout = new QHBoxLayout(this, 0, -1, "layout" );
	m_pQueryResultWindow = new KPGQueryResultWindow(this, "SQL Query/result window");
	pLayout->addWidget( m_pQueryResultWindow );
	
	setIcon( SmallIcon("openterm") );
	newDocument();
	
	connect(m_pQueryResultWindow, SIGNAL(sigUpdateTabTooltip(const QString &)),
		this, SLOT(slotUpdateTabTooltip(const QString &)));
}

KPGQueryResultChildView::~KPGQueryResultChildView()
{
}

const KURL& KPGQueryResultChildView::URL() const
{
  return m_urlSqlFile;
}

bool KPGQueryResultChildView::newDocument()
{
  m_pQueryResultWindow->setQueryTextChanged(false);
  m_urlSqlFile.setFileName(i18n("Untitled"));

  return true;
}

bool KPGQueryResultChildView::openDocument(const KURL& url, const char */*format*/ /*=0*/)
{
	QString tmpfile;
	KIO::NetAccess::download( url, tmpfile, m_pQueryResultWindow);
	
	QFile file(tmpfile);
	if( !file.open(IO_ReadOnly) )
	{ 
		QString error =  i18n("Error opening ");
		error.append(tmpfile);
		
		KMessageBox::error(this, error, "KPoGre");
		KIO::NetAccess::removeTempFile( tmpfile );
		return false;
	}
	else
	{ 
		QTextStream inStream(&file);
		while(!inStream.atEnd())
		{ 
			m_pQueryResultWindow->editQuery()->setText(inStream.read());
		}
		file.close();
	}
	
	KIO::NetAccess::removeTempFile( tmpfile );
	m_pQueryResultWindow->setQueryTextChanged(false);
	m_urlSqlFile = url;
	setCaption(url.fileName());
	return true;
}

bool KPGQueryResultChildView::saveDocument(const KURL& url, const char */*format*/ /*=0*/)
{
	QFile file(url.path());
	if(file.open(IO_WriteOnly) == true)
	{ 
		QTextStream outStream(&file);
		outStream << m_pQueryResultWindow->editQuery()->text();

 		file.flush();
		file.close();
	}
	else
	{ 
		KMessageBox::error(this, i18n("Write error. Can't create file %1").arg(url.path()));
		return false;
	}

  	m_pQueryResultWindow->setQueryTextChanged(false);
	m_urlSqlFile = url;
	setCaption(url.fileName());
  	return true;
}

const bool KPGQueryResultChildView::isModified() const 
{
	return m_pQueryResultWindow->queryTextChanged();
}

bool KPGQueryResultChildView::requestClose()
{
	if(m_pQueryResultWindow->running())
	{
		KMessageBox::sorry(this, i18n("Query is running, cannot close !"));	 
		return false;
	}
	
	if(isModified())
	{
		if(KMessageBox::questionYesNo(this, i18n("SQL script is not saved, really close window ?")) == KMessageBox::Yes) 
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	return true;
}

// Re-route signal, add pointer of this class
void KPGQueryResultChildView::slotUpdateTabTooltip(const QString &strText)
{
	emit sigUpdateTabTooltip(this, strText);
}

