//
// C++ Implementation: kpgqueryresultwindow
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgqueryresultwindow.h"


// include files for Qt
#include <qtable.h>
#include <qwidgetstack.h> 
#include <qtextcodec.h>
#include <qsplitter.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qregexp.h>

// include files for KDE
#include <kcombobox.h>
#include <kpushbutton.h>
#include <kcursor.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <klocale.h>
#include <kfinddialog.h>
#include <kfind.h>

// application specific includes
#include "kpgsqledit.h"
#include "../DbObjects/kpgconnection.h"
#include "../DbObjects/kpgserver.h"
#include "../DbObjects/kpgdatabase.h"

#define NUM_OF_RECORDS_PER_PAGE 500 // How many records are loaded to result table 


KPGQueryResultWindow::KPGQueryResultWindow(QWidget *parent, const char *name)
	: KPGQueryResultWindowBase(parent, name),
 	m_pFind(0)
{
	m_pTextEditQuery = new KPGSqlEdit(m_pFrameEditQuery, "SQL Edit");
	m_pFrameEditQueryLayout->addWidget(m_pTextEditQuery, 0, 0 );
	
	m_pTextEditQuery->setText("SELECT * FROM ;");
	m_pTextEditQuery->setSelection(14 , 0, 14, 0),
 		
	connect(m_pTextEditQuery, SIGNAL(textChanged()), this, SLOT(slotTextEditQueryTextChanged()));
	connect(m_pTextEditQuery, SIGNAL(copyAvailable(bool)), this, SLOT(slotTextEditCopyAvailable(bool)));
	connect(m_pTextEditQuery, SIGNAL(undoAvailable(bool)), this, SLOT(slotTextEditUndoAvailable(bool)));
	connect(m_pTextEditQuery, SIGNAL(redoAvailable(bool)), this, SLOT(slotTextEditRedoAvailable(bool)));
	
	m_bQueryTextChanged = false;
  
	m_ConnectionInThread.setEventReceiver(this, &m_pqxxResult);
	
	m_nTotalRows = 0;
	m_nTotalCols = 0;
	m_nFetchedRows = 0;
	
	m_queryResultActions.bEnableOpen = true;
    m_queryResultActions.bEnableSave = false; 
    m_queryResultActions.bEnableSaveAs = false;
    m_queryResultActions.bEnableExport = false;
    
	m_queryResultActions.bEnableUndo = false;
    m_queryResultActions.bEnableRedo = false;
    m_queryResultActions.bEnableCutCopy = false;
    m_queryResultActions.bEnablePaste = true;
    m_queryResultActions.bEnableClearInpArea = false;
    
    m_queryResultActions.bEnableRunQuery = false;
    m_queryResultActions.bEnableStopQuery = false;
    m_queryResultActions.bEnableFetchNext = false; 
    m_queryResultActions.bEnableFetchAll = false;
    
    m_pFrameOptions->hide();
}

KPGQueryResultWindow::~KPGQueryResultWindow()
{
	if(m_pFind != 0) delete m_pFind;
}

// Set flag indicating changed query text
void KPGQueryResultWindow::setQueryTextChanged(bool changed) 
{ 
   m_bQueryTextChanged = changed; 
   m_queryResultActions.bEnableSave = changed;
   m_queryResultActions.bEnableSaveAs = true;
}

void KPGQueryResultWindow::setEnableRunQuery(bool bEnable) 
{
    m_queryResultActions.bEnableRunQuery = bEnable;
}

void KPGQueryResultWindow::slotServerChanged(const QString& strServer)
{
    m_queryResultActions.bEnableRunQuery = false;
	emit sigRequestDatabasesList(this, strServer);
}

void KPGQueryResultWindow::slotDatabaseChanged(const QString& strDatabase)
{
    kdDebug() << k_funcinfo << "Database selected: " << strDatabase << endl;
    emit sigRequestListOfDatabaseObjectsForCodeCompletition(this, strDatabase);
    
    m_queryResultActions.bEnableRunQuery = ((m_pComboBoxDatabase->count() > 0) && 
	   (m_pTextEditQuery->text().length() > 0));
	   
	emit sigQueryResultStateChanged(this);
}

void KPGQueryResultWindow::slotTextEditQueryTextChanged()
{
	if(!m_bQueryTextChanged)
	{
		m_bQueryTextChanged = true;
		m_queryResultActions.bEnableSave = true;
		m_queryResultActions.bEnableSaveAs = true; 
	}
	
	m_queryResultActions.bEnableRunQuery = ((m_pComboBoxDatabase->count() > 0) && 
	   (m_pTextEditQuery->text().length() > 0));
	   
	m_queryResultActions.bEnableUndo = editQuery()->isUndoAvailable();
    m_queryResultActions.bEnableRedo = editQuery()->isRedoAvailable();
	   
	emit sigQueryResultStateChanged(this);
	updateTabToolTip();
}

void KPGQueryResultWindow::slotTextEditCopyAvailable(bool bCopyAvailable)
{
	m_queryResultActions.bEnableCutCopy =  bCopyAvailable;
	emit sigQueryResultStateChanged(this);
}

void KPGQueryResultWindow::slotTextEditUndoAvailable(bool bUndoAvailable)
{
	m_queryResultActions.bEnableUndo =  bUndoAvailable; 
	emit sigQueryResultStateChanged(this);  
}

void KPGQueryResultWindow::slotTextEditRedoAvailable(bool bRedoAvailable)
{
	m_queryResultActions.bEnableRedo = bRedoAvailable;   
	emit sigQueryResultStateChanged(this);
}

bool KPGQueryResultWindow::resultEmpty() const
{ 
	return m_pTableResult->numRows() == 0; 
}

KPGDatabase *KPGQueryResultWindow::lookupDatabase(QListViewItem * pItem)
{
	// get selected server and database
	QString strServer(m_pComboBoxServer->currentText());
	QString strDatabase(m_pComboBoxDatabase->currentText());
		
	if(strServer.length() == 0)
	{
		kdError() << k_funcinfo << " Server is not selected " << endl;
		return 0;
	}
		
	if(strDatabase.length() == 0)
	{
		kdError() << k_funcinfo << " Database is not selected " << endl;
		return 0;
	}
		
	while(pItem)
	{
		if((static_cast <KPGTreeItem *> (pItem))->type() == KPGTreeItem::nodeServer)
		{
            KPGServer *pServer = static_cast <KPGServer *> (pItem);
            if(pServer->text(0) != strServer)
            { 
                pItem = pItem->nextSibling(); continue; }
                        
                //--- traverse list of connected databases
                QListViewItem * pItem2 = pServer->firstChild();
                
                while(pItem2)
                {
                if((static_cast <KPGTreeItem *> (pItem2))->type() == KPGTreeItem::nodeDatabase)
                {
                    KPGDatabase *pDatabase = static_cast <KPGDatabase *> (pItem2);
                    if(pDatabase->text(0) != strDatabase)
                    { 
                        pItem2 = pItem2->nextSibling(); continue; 
                    }
                
                    if(pDatabase->connection())
                    { 
                        return pDatabase;      
                    }
                    else
                    {
                        kdError() << k_funcinfo << " Database is disconected " << endl;
                        return 0;
                    }
                }     
                pItem2 = pItem2->nextSibling();
			}  
		}
		pItem = pItem->nextSibling();
	}
	
	return 0;
}

void KPGQueryResultWindow::runQuery(KPGDatabase *pDatabase)
{
	// clear previous result
	m_pTableResult->setNumRows(0);
	m_pTableResult->setNumCols(0);
	m_pTextEditResult->clear();
	m_pTextLabelExecutionTime->setText("");
	m_pqxxResult.clear(); // free memory
		
	int iTransType = m_pComboBoxTransaction->currentItem();
	KPGConnection::eTransType transType = KPGConnection::eTransNormal;
	switch(iTransType)
	{
		case 0: transType = KPGConnection::eTransNormal;
				break;
				
		case 1: transType = KPGConnection::eTransRobust;
				break;
				
		case 2: transType = KPGConnection::eTransNone;
				break;
	}
	
	QString strConectionString( );
	
	m_pTextEditQuery->setEnabled(false);
	m_pComboBoxServer->setEnabled(false);
	m_pComboBoxDatabase->setEnabled(false);
	m_pComboBoxTransaction->setEnabled(false);
	emit sigSetTerminalIcon(this, 1); // red terminal icon
	((QWidget *) parent())->setCursor(KCursor::waitCursor());
		
	m_queryResultActions.bEnableOpen = false;
    m_queryResultActions.bEnableSave = false;
    m_queryResultActions.bEnableSaveAs = false;
    m_queryResultActions.bEnableExport = false;
	
	m_queryResultActions.bEnableRunQuery = false;
    m_queryResultActions.bEnableStopQuery = true;
    m_queryResultActions.bEnableFetchNext = false; 
    m_queryResultActions.bEnableFetchAll = false;
    
	emit sigQueryResultStateChanged(this);
	
	// Run connection in thread
	m_ConnectionInThread.disconnectFromServer(); // close connection if any
	
	try
	{
		m_ConnectionInThread.connectToServer(pDatabase->connection()->options());
	}
	catch (const std::exception &e)
    {
        kdError() << "Failed to open connection " << e.what() << endl;
        KMessageBox::sorry(this, e.what());
        return;
    }
    
    QString strSql;
    
    if(m_pCheckBoxExplain->isChecked())
    {
    	strSql.append("EXPLAIN ");
    
    	if(m_pCheckBoxAnalyze->isChecked())
    		strSql.append("ANALYZE ");
    		
		if(m_pCheckBoxVerbose->isChecked())
			strSql.append("VERBOSE ");
    }
    
    strSql.append(m_pTextEditQuery->text());
    
	m_ConnectionInThread.setSQL(strSql, transType);
	m_ConnectionInThread.start();
}

void KPGQueryResultWindow::customEvent(QCustomEvent *pEvent)
{
	KPGQueryResultEvent *pQueryResultEvent = static_cast <KPGQueryResultEvent *> (pEvent);
	
	//kdDebug() << QString("Result size is: %1").arg(m_pqxxResult.size()) << endl;
	
	m_nFetchedRows = 0;
		
	// Get codec for connection client encoding
 	m_pTextCodec = pQueryResultEvent->connection()->textCodec();
			
	// Display result
	if(m_pqxxResult.size() > 0)
	{
		displayExecutionTime(pQueryResultEvent->executionMiliseconds());   
		displayResult(); 
		m_pWidgetStackResults->raiseWidget(0);
	}
	else
	{
		m_nTotalRows = 0;
		m_nTotalCols = 0;
	
		if(pQueryResultEvent->error().isEmpty() == false)
		{
			QString strErrorText(pQueryResultEvent->error());
			m_pTextEditResult->setText(strErrorText);
			
			if(strErrorText.contains("syntax error at or near") > 0)
			{
				// Parse error message and obtain number of character position
				QRegExp patternNumber = QRegExp("\\d+\n", false);
				int iPos = patternNumber.search(strErrorText, 0);
				
				if(iPos > 0)
				{
					QString strNumber = strErrorText.mid(iPos);
					kdDebug() << "Error at character: " << strNumber << endl;
					
					bool bConversionOk;
					int iErrorPosition = strNumber.toInt(&bConversionOk);
					
					if(bConversionOk)
					{
						// Place cursor in editor to error position
						m_pTextEditQuery->highlightCharacter(iErrorPosition);
						m_pTextEditQuery->setFocus();
					}
				}
			}
		}
		else
		{
			displayExecutionTime(pQueryResultEvent->executionMiliseconds());      
			
			if(pQueryResultEvent->connection()->notifications().count() > 0)
			{
				m_pTextEditResult->setText(pQueryResultEvent->connection()->notifications().join(""));
				pQueryResultEvent->connection()->clearNotifications();
			}
			else  
			{
				m_pTextEditResult->setText(i18n("Operation successfull, no rows returned"));
			}
		}
		m_pWidgetStackResults->raiseWidget(1); 
	}
	
	m_pTextEditQuery->setEnabled(true);
	m_pComboBoxServer->setEnabled(true);
	m_pComboBoxDatabase->setEnabled(true);
	m_pComboBoxTransaction->setEnabled(true);
	emit sigSetTerminalIcon(this, 2); // blue terminal icon
	((QWidget *) parent())->setCursor(KCursor::arrowCursor());
	
	m_ConnectionInThread.wait();
	
	m_queryResultActions.bEnableOpen = true;
    m_queryResultActions.bEnableSave = queryTextChanged();
    m_queryResultActions.bEnableSaveAs = true;
    m_queryResultActions.bEnableExport = (resultEmpty()) ?  false : true;
    
    m_queryResultActions.bEnableUndo = editQuery()->isUndoAvailable();
    m_queryResultActions.bEnableRedo = editQuery()->isRedoAvailable();
    m_queryResultActions.bEnableCutCopy = editQuery()->hasSelectedText();
    m_queryResultActions.bEnablePaste = true;
    m_queryResultActions.bEnableClearInpArea = (editQuery()->length() > 0) ? true : false;
	
	m_queryResultActions.bEnableRunQuery = true;
    m_queryResultActions.bEnableStopQuery = false;
    
    m_queryResultActions.bEnableFetchNext = (fetchedAllRows()) ? false : true; 
    m_queryResultActions.bEnableFetchAll = (fetchedAllRows()) ? false : true;
	
	emit sigQueryResultStateChanged(this);
}

void KPGQueryResultWindow::stopQuery()
{
	if(m_ConnectionInThread.running ())
	{
		m_ConnectionInThread.terminate();
		m_ConnectionInThread.wait();
		m_ConnectionInThread.leaveConnection();
		
		m_pTextEditResult->setText(i18n("Operation terminated"));
		m_pWidgetStackResults->raiseWidget(1); 
		
		m_pTextEditQuery->setEnabled(true);
		m_pComboBoxServer->setEnabled(true);
		m_pComboBoxDatabase->setEnabled(true);
		m_pComboBoxTransaction->setEnabled(true);
		emit sigSetTerminalIcon(this, 0); // black terminal icon
		m_bQueryTextChanged = false;
		
		m_queryResultActions.bEnableOpen = true;
        m_queryResultActions.bEnableSave = queryTextChanged();
        m_queryResultActions.bEnableSaveAs = true; 
        m_queryResultActions.bEnableExport = false;
        
        m_queryResultActions.bEnableUndo = editQuery()->isUndoAvailable();
        m_queryResultActions.bEnableRedo = editQuery()->isRedoAvailable();
        m_queryResultActions.bEnableCutCopy = editQuery()->hasSelectedText();
        m_queryResultActions.bEnablePaste = true;
        m_queryResultActions.bEnableClearInpArea = (editQuery()->length() > 0) ? true : false;
        
        m_queryResultActions.bEnableRunQuery = true;
        m_queryResultActions.bEnableStopQuery = false;
        m_queryResultActions.bEnableFetchNext = false; 
        m_queryResultActions.bEnableFetchAll = false;
		
		m_pqxxResult.clear(); // free memory
		m_nTotalRows = 0;
		m_nTotalCols = 0;
		
		emit sigQueryResultStateChanged(this);
		((QWidget *) parent())->setCursor(KCursor::arrowCursor());
	} 
}


void KPGQueryResultWindow::displayExecutionTime(int iMiliseconds)
{
    int hour = 0; int min = 0; int sec = 0;
    int milis = iMiliseconds;
    while(milis >= 1000)
    {
      milis -= 1000;
      sec++;
    }
    while(sec >= 60)
    {
      sec -=60;
      min++;
    }
    while(min >= 60)
    {
      min -= 60;
      hour++;
    }
    
    QString execTime;
    if(hour > 0) execTime.append(QString("%1 h :").arg(hour));
    execTime.append(QString("%1 m : ").arg(min));
    execTime.append(QString("%1 s : ").arg(sec));
    execTime.append(QString("%1 ms").arg(milis));
      
    m_pTextLabelExecutionTime->setText(execTime);
}

// Display SQL result
void KPGQueryResultWindow::displayResult()
{
	bool bAllRowsFetched = true;
	
	// Fill output area with query result
	m_nTotalRows = m_pqxxResult.size();
	m_nTotalCols = m_pqxxResult.columns();
		
	m_pTableResult->setNumRows(min(m_nTotalRows, NUM_OF_RECORDS_PER_PAGE));
	m_pTableResult->setNumCols(m_nTotalCols);
	
	// Field names
	QHeader* hHeader = m_pTableResult->horizontalHeader();
	
	for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
	{
		hHeader->setLabel(nCol, m_pqxxResult.column_name(nCol));
	}
	
	
	// Data
	for(unsigned int nRow = 0; nRow < m_nTotalRows; nRow++)
	{
		for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
		{
			QString strValue(m_pTextCodec->toUnicode(m_pqxxResult[nRow][nCol].c_str()));
			m_pTableResult->setText(nRow, nCol, strValue);
		}
		
		m_nFetchedRows++;
		if(nRow + 1 >= NUM_OF_RECORDS_PER_PAGE)
		{
			bAllRowsFetched = false;
			break;
		}
	}
	
	for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
	{
		m_pTableResult->adjustColumn(nCol);
	}
	
	if(bAllRowsFetched) m_pqxxResult.clear(); // free memory
}

// Fetch next X rows from large result
void KPGQueryResultWindow::fetchNext()
{
	unsigned int nRows = m_nTotalRows - m_nFetchedRows;
	nRows = min(nRows, NUM_OF_RECORDS_PER_PAGE);
	m_pTableResult->setNumRows(nRows + m_pTableResult->numRows());
	
	// Data
	unsigned int nRow = m_nFetchedRows;
	unsigned int nFetchedRows = 0;
	
	for(; nRow < m_nTotalRows; nRow++)
	{
		for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
		{
			QString strValue(m_pTextCodec->toUnicode(m_pqxxResult[nRow][nCol].c_str()));
			m_pTableResult->setText(nRow, nCol, strValue);
		}
		
		m_nFetchedRows++;
		nFetchedRows++;
		if(nFetchedRows >= NUM_OF_RECORDS_PER_PAGE)
		{
			m_queryResultActions.bEnableFetchNext = true; 
            m_queryResultActions.bEnableFetchAll = true;
            break;
		}
	}
	
	if(nRow == m_nTotalRows)
	{
		m_queryResultActions.bEnableFetchNext = false; 
        m_queryResultActions.bEnableFetchAll = false;
		m_pqxxResult.clear(); // free memory
	}
	
	emit sigQueryResultStateChanged(this);
}
  
// Fetch all rows from large result  
void KPGQueryResultWindow::fetchAll()
{
	unsigned int nRows = m_nTotalRows - m_nFetchedRows;
	
	if(nRows > 10000)
	{ 
		if( KMessageBox::questionYesNo(this, QString("There are %1 rows in result. Fething them may take long time and much anmount of memory. Continue ?").arg(nRows)) != KMessageBox::Yes )
		{
			return;
		}
	}
	
	((QWidget *) parent())->setCursor(KCursor::waitCursor());
	m_pTableResult->setNumRows(nRows + m_pTableResult->numRows());
	
	// Data
	unsigned int nRow = m_nFetchedRows;
	for(; nRow < m_nTotalRows; nRow++)
	{
		for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
		{
			QString strValue(m_pTextCodec->toUnicode(m_pqxxResult[nRow][nCol].c_str()));
			m_pTableResult->setText(nRow, nCol, strValue);
		}
		
		m_nFetchedRows++;
	}
	
	((QWidget *) parent())->setCursor(KCursor::arrowCursor());
	
	m_queryResultActions.bEnableFetchNext = false; 
    m_queryResultActions.bEnableFetchAll = false;
    emit sigQueryResultStateChanged(this);
    
	m_pqxxResult.clear(); // free memory
}  

void KPGQueryResultWindow::updateTabToolTip()
{
	QString strText = m_pTextEditQuery->text().left(150);
	
	emit sigUpdateTabTooltip(strText);
}

void KPGQueryResultWindow::slotExplainToggled(bool bToggled)
{
	m_pCheckBoxAnalyze->setEnabled(bToggled);
	m_pCheckBoxVerbose->setEnabled(bToggled);
}

void KPGQueryResultWindow::slotButtonOptionsToggleg(bool bToggled)
{
	if(bToggled)
	{
		m_pFrameOptions->show();
	}
	else
	{
		m_pFrameOptions->hide();
	}
}

// Set list of objects for code completition to SQL editor 
void KPGQueryResultWindow::setListOfObjectsForCodeCompletition(const KPGOidNameList &listOfCodeCompletitionObjects)
{
   m_pTextEditQuery->setListOfObjectsForCodeCompletition(listOfCodeCompletitionObjects);
}

// Clear list of objects for code completition to SQL editor 
void KPGQueryResultWindow::clearListOfObjectsForCodeCompletition()
{
   m_pTextEditQuery->clearListOfObjectsForCodeCompletition(); 
}

// Find first occurence of text in SQL query editor
void  KPGQueryResultWindow::findInQueryFirst(QStringList &listOfSearchHistory)
{
    m_pTextEditQuery->findFirst(listOfSearchHistory);
}

// Find next occurence of text in SQL query editor
void  KPGQueryResultWindow::findInQueryNext()
{
    m_pTextEditQuery->findNext();
}

// Find first occurence of text in result table
void KPGQueryResultWindow::findInResultFirst(QStringList &listOfSearchHistory)
{
    KFindDialog dlg(this, "", 0, listOfSearchHistory, false);
    dlg.setHasCursor(false);
        
    int c = dlg.exec();
    
    if(c != QDialog::Accepted)
        return;

    listOfSearchHistory = dlg.findHistory();
        
    
    if(m_pFind != 0) delete m_pFind;
    m_pFind = new KFind(dlg.pattern(), dlg.options(), this);

    // Connect highlight signal to code which handles highlighting
    // of found text.
    connect( m_pFind, SIGNAL( highlight( const QString &, int, int ) ),
             this, SLOT( slotHighlight( const QString &, int, int ) ) );
             
    // Connect findNext signal - called when pressing the button in the dialog
    connect( m_pFind, SIGNAL( findNext() ), this, SLOT( slotFindInResultNext() ) );

    m_iRowToSearch = (m_pFind->options() & KFindDialog::FindBackwards) ? m_pTableResult->numRows() - 1 : 0;
    m_iColToSearch = (m_pFind->options() & KFindDialog::FindBackwards) ? m_pTableResult->numCols() - 1 : 0;
    m_pFind->setData(m_pTableResult->text(m_iRowToSearch, m_iColToSearch));

    slotFindInResultNext();
}

// Find next occurence of text in result table
void KPGQueryResultWindow::findInResultNext()
{
    m_iRowToSearch = m_pTableResult->currentRow();
	m_iColToSearch = m_pTableResult->currentColumn();
	if(m_iRowToSearch < 0) return;
	if(m_iColToSearch < 0) return;
	
    slotFindInResultNext();
}

// Find next occurence of text
void KPGQueryResultWindow::slotFindInResultNext()
{
	if (!m_pFind) // shouldn't be called before find is activated
        return;

    KFind::Result res = KFind::NoMatch;
    while ( res == KFind::NoMatch &&
    		((m_pFind->options() & KFindDialog::FindBackwards) ? 
    				(m_iRowToSearch >= 0) : (m_iRowToSearch < m_pTableResult->numRows())
    		) 
          ) 
    	{
    		//kdDebug() << "searching  1:" << m_iParaToSearch << " 2: " << m_iEndPara << " 3: " << m_iStartPara << endl;
        	
        	Q_ASSERT(m_iRowToSearch >= 0);
            Q_ASSERT(m_iRowToSearch < m_pTableResult->numRows());
            Q_ASSERT(m_iColToSearch >= 0);
            Q_ASSERT(m_iColToSearch < m_pTableResult->numCols());
            	
        	if(m_pFind->needData()) 
        	{
            	m_pFind->setData(m_pTableResult->text(m_iRowToSearch, m_iColToSearch));
        	}
        
        	// Let KFind inspect the text fragment, and display a dialog if a match is found
        	res = m_pFind->find();
        
        	if( res == KFind::NoMatch ) 
        	{
            	if((m_pFind->options() & KFindDialog::FindBackwards))
            	{
            		if(m_iColToSearch == 0)
            		{
            			m_iColToSearch = m_pTableResult->numCols() - 1;
            			m_iRowToSearch--;
            		}
            		else
            		{
            			m_iColToSearch--;
            		}
            	}
            	else
            	{
            		if(m_iColToSearch == m_pTableResult->numCols() - 1)
            		{
            			m_iColToSearch = 0;
            			m_iRowToSearch++;
            		}
            		else
            		{
            			m_iColToSearch++;
            		}
            	}
        	}
        	//kdDebug() << "2searching  1:" << m_iParaToSearch << " 2: " << m_iEndPara << " 3: " << m_iStartPara << endl;
		}

    if( res == KFind::NoMatch ) 
    { // i.e. at end
        m_pFind->displayFinalDialog();
        m_pFind->resetCounts();
        m_pTableResult->removeSelection(0);
    }
}

// Highligth found text
void KPGQueryResultWindow::slotHighlight( const QString &, int, int)
{
    //kdDebug() << "highlight: " << index << " " << length << endl;
    m_pTableResult->setCurrentCell(m_iRowToSearch, m_iColToSearch);
}


// Replace text in SQL query editor
void KPGQueryResultWindow::replaceInQuery(QStringList &listOfSearchHistory, QStringList &listOfReplacementHistory)
{
	m_pTextEditQuery->replace(listOfSearchHistory, listOfReplacementHistory);
}
