//
// C++ Interface: kpgqueryresultwindow
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef KPGQUERYRESULTWINDOW_H
#define KPGQUERYRESULTWINDOW_H

#include "kpgqueryresultwindowbase.h"

#include "../kpgconnectioninthread.h"

#include "../kpgactionsstates.h"

class QListViewItem;
class KFind;
class KPGSqlEdit;
class KCompletionBox;
class KPGDatabase;

/**
SQL Query / result window. Allow enter SQL query, run it and display the result.

	@author Lumir Vanek <lvanek@users.sourceforge.net>
*/
class KPGQueryResultWindow : public KPGQueryResultWindowBase
{
	Q_OBJECT
public: 
	KPGQueryResultWindow(QWidget *parent=0, const char *name=0);
	~KPGQueryResultWindow();

    // Get actions state
    const QueryResultActions & queryResultActions() const { return m_queryResultActions; }
    
    // Set state of flag for action Run query
    void setEnableRunQuery(bool bEnable);
    
  	KPGSqlEdit * editQuery() const { return m_pTextEditQuery; }
  	
  	// Returns QTable with result
  	QTable *tableResult() const { return m_pTableResult; }

	// Is SQL query changed after last open/save ?
	bool queryTextChanged() const { return m_bQueryTextChanged; }
	
	// Set flag indicating changed query text
	void setQueryTextChanged(bool);
	
	// Signalize, if result contain at least one row
	bool resultEmpty() const;
  
  	// Is all rows fetched from resultset do m_pTableResult ?
  	bool fetchedAllRows() const { return m_nTotalRows == m_nFetchedRows;  }
  
  	// Lookup KPGDatabase for selected server and database in combo boxes 
  	KPGDatabase * lookupDatabase(QListViewItem *);
  
	// Run SQL query
	void runQuery(KPGDatabase *);
	
	// Terminate thread that run SQL query
	void stopQuery();
	
	// Returns TRUE if the connection thread is running; otherwise returns FALSE.
	bool running() const { return  m_ConnectionInThread.running(); }
	
	// Fetch next X rows from large result
	void fetchNext();
	
	// Fetch all rows from large result  
	void fetchAll();
	
	// Set list of objects for code completition to SQL editor 
	void setListOfObjectsForCodeCompletition(const KPGOidNameList &);
	
	// Clear list of objects for code completition to SQL editor 
	void clearListOfObjectsForCodeCompletition();
	
	// Find first occurence of text in SQL query editor
	void findInQueryFirst(QStringList &);
	
	// Find next occurence of text in SQL query editor
	void findInQueryNext();
	
	// Find first occurence of text in result table
	void findInResultFirst(QStringList &);
		
	// Find next occurence of text in result table
	void findInResultNext();
	
	// Replace text in SQL query editor
	void replaceInQuery(QStringList &, QStringList &);
	
protected:
   
    // Receive event from KPGConnectionInThread
    virtual void customEvent(QCustomEvent *);

    // Display SQL result
    void displayResult();  
    
    // Display query execution time
    void displayExecutionTime(int);
  
  	void updateTabToolTip();
  
public slots:

	void slotDatabaseChanged(const QString&);
  
protected slots:
    
	void slotServerChanged(const QString &);
    
	// Called, when user change text in SQL input area. Mark document as modified.
   	void slotTextEditQueryTextChanged();

    // Called, when user Copy action availability is changed in SQL input area
	void slotTextEditCopyAvailable(bool);

	// Called, when Undo action availability is changed in SQL input area
	void slotTextEditUndoAvailable(bool);

	// Called, when Redo action availability is changed in SQL input area
	void slotTextEditRedoAvailable(bool);
	
	virtual void slotExplainToggled(bool);
	
	virtual void slotButtonOptionsToggleg(bool);
	
	// Find next occurence of text
    void slotFindInResultNext();
    
    // Highligth found text
    void slotHighlight( const QString &, int, int);
	
private:
	int max(int a, int b) { return a > b ? a : b; }
	int min(int a, int b) { return a < b ? a : b; }
		
signals:

	// Fired, when info about databases list is needed
	void sigRequestDatabasesList(KPGQueryResultWindow *, const QString&);
	
	// Fired, when database is selected and list of object for code completition is needed
	void sigRequestListOfDatabaseObjectsForCodeCompletition(KPGQueryResultWindow *, const QString&);
				
	// Fired for request to change terminal icon in QTabWidget 
	void sigSetTerminalIcon(KPGQueryResultWindow *, int);
		
	// Fired when SQL query started or finished
	void sigQueryResultStateChanged(KPGQueryResultWindow *);
	
	// Fired when need update tooltip text
	void sigUpdateTabTooltip(const QString &);
		  
protected:
	// Editor for SQL
	KPGSqlEdit *m_pTextEditQuery;

    // State of actions - enable or disabled
    QueryResultActions m_queryResultActions;
    
    // Codec for encode/decode strings from/to database
    QTextCodec *m_pTextCodec;
	
	bool m_bQueryTextChanged;
	
	// Thread that run queries on background
	KPGConnectionInThread m_ConnectionInThread;
	
	// Resultset to display
	pqxx::result m_pqxxResult;
	
	unsigned int m_nTotalRows; // Number of rows in result
	unsigned int m_nFetchedRows; // Number of fetched rows from result
	unsigned int m_nTotalCols; // Number of columns in result
	
private:
	// A generic implementation of the "find" function
	KFind *m_pFind;
	
	int m_iRowToSearch;
	int m_iColToSearch;

};

#endif
