//
// C++ Implementation: kpg3stepswizard
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpg3stepswizard.h"

// include files for KDE
#include <kdebug.h>
#include <ktextedit.h>
#include <kcursor.h>

// application specific includes
#include "../kpogreview.h"
#include "../kpgconfiguration.h"
#include "../QueryResult/kpgsqleditorsettings.h"
#include "../QueryResult/kpgsyntaxhighlighter.h"
#include "../DbObjects/kpgconnection.h"
#include "../DbObjects/kpgdatabase.h"


KPG3StepsWizard::KPG3StepsWizard(QWidget *parent, const char *name)
 : KPG3StepsWizardBase(parent, name)
{
	m_pConnection = 0;
    m_pTextEditSQL->setFont( KPoGreView::configuration()->sqleditor()->font() );   
	m_pSyntaxHighlighter = new KPGSyntaxHighlighter(m_pTextEditSQL);

	m_pSyntaxHighlighter->setColorDefaultText( KPoGreView::configuration()->sqleditor()->colorDefaultText() );
	m_pSyntaxHighlighter->setColorKeyWords( KPoGreView::configuration()->sqleditor()->colorKeyWords() );
	m_pSyntaxHighlighter->setColorDataTypes( KPoGreView::configuration()->sqleditor()->colorDataTypes() );
	m_pSyntaxHighlighter->setColorOperators( KPoGreView::configuration()->sqleditor()->colorOperators() );
	m_pSyntaxHighlighter->setColorQuotedStrings( KPoGreView::configuration()->sqleditor()->colorQuotedStrings() );
	m_pSyntaxHighlighter->setColorNumbers( KPoGreView::configuration()->sqleditor()->colorNumbers() );
    m_pSyntaxHighlighter->setColorComments( KPoGreView::configuration()->sqleditor()->colorComments() );
	
	m_pSyntaxHighlighter->rehighlight();	
		
	setNextEnabled(WizardPage1, false);
}


KPG3StepsWizard::~KPG3StepsWizard()
{
	delete m_pSyntaxHighlighter;
}

// Set connection for wizard operation
void KPG3StepsWizard::setConnection(KPGConnection * pConnection)
{
	m_pConnection = pConnection;
}

// Enable/disable next button for 1st page
void KPG3StepsWizard::slotEnableNextButton1(bool bEnable)
{
	setNextEnabled(WizardPage1, bEnable);
}

// Run SQL query - as final Wizard operation
void KPG3StepsWizard::runQuery(bool bWithTransaction) 
{ 
	// Run SQL statement
	if(m_pConnection == 0)
	{
		kdError() << "KPG3StepsWizard::runQuery - Database not connected to server !" << endl;
		m_pTextEditResult->setText("Database not connected to server !");
	}
	else
	{ 
		try
		{ 
			setCursor(KCursor::waitCursor());
			
			pqxx::result R = m_pConnection->runQuery(m_pTextEditSQL->text(), bWithTransaction ? KPGConnection::eTransNormal : KPGConnection::eTransNone);
			
			if(m_pConnection->notifications().count() > 0)
			{
			m_pTextEditResult->setText(m_pConnection->notifications().join("") + i18n("\nOperation successfull"));
			m_pConnection->clearNotifications();
			}
			else
			m_pTextEditResult->setText(i18n("Operation successfull"));
			
			setFinishEnabled(WizardPage3, true);
			setCursor(KCursor::arrowCursor());
		}
		catch (const std::exception &e)
		{
			m_pTextEditResult->setText(e.what());
			setCursor(KCursor::arrowCursor());
		} 
	}
}

#include "kpg3stepswizard.moc"
